@@ -24,7 +24,7 @@
 #include "ardour_dialog.h"
 #include "keyboard.h"
 #include "ardour_ui.h"
-
+#include "splash.h"
 
 ArdourDialog::ArdourDialog (string title, bool modal, bool use_seperator)
 	: Dialog (title, modal, use_seperator)
@@ -34,6 +34,15 @@ ArdourDialog::ArdourDialog (string title, bool modal, bool use_seperator)
 	set_type_hint(Gdk::WINDOW_TYPE_HINT_DIALOG);
 }
 
+ArdourDialog::ArdourDialog (Gtk::Window& parent, string title, bool modal, bool use_seperator)
+	: Dialog (title, parent, modal, use_seperator)
+{
+	session = 0;
+
+	set_type_hint(Gdk::WINDOW_TYPE_HINT_DIALOG);
+	set_position (Gtk::WIN_POS_CENTER_ON_PARENT);
+}
+
 ArdourDialog::~ArdourDialog ()
 {
 }
@@ -57,3 +66,17 @@ ArdourDialog::on_unmap ()
 {
 	Dialog::on_unmap ();
 }
+
+void
+ArdourDialog::on_show ()
+{
+	// never allow the splash screen to obscure any dialog
+
+	Splash* spl = Splash::instance();
+
+	if (spl) {
+		spl->pop_back ();
+	}
+
+	Dialog::on_show ();
+}
