@@ -73,6 +73,7 @@ class Mixer_UI;
 class ConnectionEditor;
 class RouteParams_UI;
 class About;
+class Splash;
 class AddRouteDialog;
 class LocationUI;
 class ThemeManager;
@@ -97,6 +98,7 @@ namespace ALSA {
 #define FRAME_NAME "BaseFrame"
 
 extern sigc::signal<void>  ColorsChanged;
+extern sigc::signal<void>  DPIReset;
 
 class ARDOUR_UI : public Gtkmm2ext::UI
 {
@@ -109,8 +111,12 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	
 	void show_splash ();
 	void hide_splash ();
+
+	void show_about ();
+	void hide_about ();
 	
-	int load_session (const Glib::ustring & path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
+	void idle_load (const Glib::ustring& path);
+	int load_session (const Glib::ustring& path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
 	bool session_loaded;
 	int build_session (const Glib::ustring& path, const Glib::ustring& snapshot, 
 			   uint32_t ctl_chns, 
@@ -132,8 +138,11 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 		_will_create_new_session_automatically = yn;
 	}
 
-	bool get_session_parameters (Glib::ustring path, bool have_engine = false, bool should_be_new = false);
-	gint cmdline_new_session (string path);
+	bool get_session_parameters (bool have_engine = false, bool should_be_new = false);
+	void parse_cmdline_path (const Glib::ustring& cmdline_path, Glib::ustring& session_name, Glib::ustring& session_path, bool& existing_session);
+	int  load_cmdline_session (const Glib::ustring& session_name, const Glib::ustring& session_path, bool& existing_session);
+	int  build_session_from_nsd (const Glib::ustring& session_name, const Glib::ustring& session_path);
+	bool ask_about_loading_existing_session (const Glib::ustring& session_path);
 	int  unload_session (bool hide_stuff = false);
 	void close_session(); 
 
@@ -186,6 +195,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void do_transport_locate (nframes_t position);
 	void halt_on_xrun_message ();
+	void xrun_handler (nframes_t);
+	void create_xrun_marker (nframes_t);
 
 	AudioClock primary_clock;
 	AudioClock secondary_clock;
@@ -218,9 +229,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void set_native_file_header_format (ARDOUR::HeaderFormat sf);
 	void set_native_file_data_format (ARDOUR::SampleFormat sf);
 
-	void set_keybindings_path (std::string path);
-	void save_keybindings ();
-
 	void setup_profile ();
 	void setup_theme ();
 
@@ -232,6 +240,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_clocking ();
 	void toggle_auto_play ();
 	void toggle_auto_input ();
+	void toggle_punch ();
+	void unset_dual_punch ();
+	bool ignore_dual_punch;
 	void toggle_punch_in ();
 	void toggle_punch_out ();
 	void toggle_auto_return ();
@@ -584,10 +595,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void connect_dependents_to_session (ARDOUR::Session *);
 	void we_have_dependents ();
 	
-	std::string keybindings_path;
-	std::string user_keybindings_path;
-
-	void setup_keybindings ();
 	void setup_session_options ();
 	
 	guint32  last_key_press_time;
@@ -639,7 +646,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void count_recenabled_streams (ARDOUR::Route&);
 
 	About* about;
+	Splash* splash;
+	void pop_back_splash ();
 	bool shown_flag;
+
 	/* cleanup */
 
 	Gtk::MenuItem *cleanup_item;
@@ -660,7 +670,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void disk_overrun_handler ();
 	void disk_underrun_handler ();
 
+	void session_dialog (std::string);
 	int pending_state_dialog ();
+	int sr_mismatch_dialog (nframes_t, nframes_t);
 	
 	void disconnect_from_jack ();
 	void reconnect_to_jack ();
@@ -701,6 +713,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_VerifyRemoveLastCapture();
 	void toggle_PeriodicSafetyBackups();
 	void toggle_StopRecordingOnXrun();
+	void toggle_CreateXrunMarker();
 	void toggle_StopTransportAtEndOfSession();
 	void toggle_GainReduceFastTransport();
 	void toggle_LatchedSolo();
@@ -712,6 +725,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_only_copy_imported_files ();
 	void toggle_ShowTrackMeters ();
 	void toggle_use_narrow_ms();
+	void toggle_rubberbanding_snaps_to_grid ();
+	void toggle_auto_analyse_audio ();
+	void toggle_TapeMachineMode();
 
 	void mtc_port_changed ();
 	void map_solo_model ();
@@ -734,7 +750,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_control_protocol (ARDOUR::ControlProtocolInfo*);
 	void toggle_control_protocol_feedback (ARDOUR::ControlProtocolInfo*, const char* group_name, std::string action_name);
 
-	bool can_save_keybindings;
 	bool first_idle ();
 
 	void no_memory_warning ();
@@ -747,7 +762,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 			      Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, Glib::RefPtr<Gtk::TextBuffer::Tag> mtag, const char *msg);
 	Gtk::Label status_bar_label;
 	Gtk::ToggleButton error_log_button;
-	Gtk::MessageDialog* loading_dialog;
+	
+	void loading_message (const std::string& msg);
+	void end_loading_messages ();
 
 	void platform_specific ();
 	void platform_setup ();
