@@ -30,6 +30,7 @@
 #include "ardour_ui.h"
 #include "actions.h"
 #include "gui_thread.h"
+#include "public_editor.h"
 
 #include "i18n.h"
 
@@ -81,6 +82,12 @@ ARDOUR_UI::toggle_denormal_protection ()
 	ActionManager::toggle_config_state ("options", "DenormalProtection", &Configuration::set_denormal_protection, &Configuration::get_denormal_protection);
 }
 
+void
+ARDOUR_UI::toggle_only_copy_imported_files ()
+{
+	ActionManager::toggle_config_state ("options", "OnlyCopyImportedFiles", &Configuration::set_only_copy_imported_files, &Configuration::get_only_copy_imported_files);
+}
+
 void
 ARDOUR_UI::set_native_file_header_format (HeaderFormat hf)
 {
@@ -380,6 +387,52 @@ ARDOUR_UI::toggle_session_auto_loop ()
 	}
 }
 
+void
+ARDOUR_UI::unset_dual_punch ()
+{
+	Glib::RefPtr<Action> action = ActionManager::get_action ("Transport", "TogglePunch");
+	
+	if (action) {
+		Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(action);
+		if (tact) {
+			ignore_dual_punch = true;
+			tact->set_active (false);
+			ignore_dual_punch = false;
+		}
+	}
+}
+
+void
+ARDOUR_UI::toggle_punch ()
+{
+	if (ignore_dual_punch) {
+		return;
+	}
+
+	Glib::RefPtr<Action> action = ActionManager::get_action ("Transport", "TogglePunch");
+
+	if (action) {
+
+		Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(action);
+
+		if (!tact) {
+			return;
+		}
+
+		/* drive the other two actions from this one */
+
+		Glib::RefPtr<Action> in_action = ActionManager::get_action ("Transport", "TogglePunchIn");
+		Glib::RefPtr<Action> out_action = ActionManager::get_action ("Transport", "TogglePunchOut");
+
+		if (in_action && out_action) {
+			Glib::RefPtr<ToggleAction> tiact = Glib::RefPtr<ToggleAction>::cast_dynamic(in_action);
+			Glib::RefPtr<ToggleAction> toact = Glib::RefPtr<ToggleAction>::cast_dynamic(out_action);
+			tiact->set_active (tact->get_active());
+			toact->set_active (tact->get_active());
+		}
+	}
+}
+
 void
 ARDOUR_UI::toggle_punch_in ()
 {
@@ -458,6 +511,18 @@ ARDOUR_UI::toggle_StopRecordingOnXrun()
 	ActionManager::toggle_config_state ("options", "StopRecordingOnXrun", &Configuration::set_stop_recording_on_xrun, &Configuration::get_stop_recording_on_xrun);
 }
 
+void
+ARDOUR_UI::toggle_CreateXrunMarker()
+{
+	ActionManager::toggle_config_state ("options", "CreateXrunMarker", &Configuration::set_create_xrun_marker, &Configuration::get_create_xrun_marker);
+}
+
+void
+ARDOUR_UI::toggle_sync_order_keys ()
+{
+	ActionManager::toggle_config_state ("options", "SyncEditorAndMixerTrackOrder", &Configuration::set_sync_all_route_ordering, &Configuration::get_sync_all_route_ordering);
+}
+
 void
 ARDOUR_UI::toggle_StopTransportAtEndOfSession()
 {
@@ -494,6 +559,36 @@ ARDOUR_UI::toggle_SecondaryClockDeltaEditCursor()
 	ActionManager::toggle_config_state ("options", "SecondaryClockDeltaEditCursor", &Configuration::set_secondary_clock_delta_edit_cursor, &Configuration::get_secondary_clock_delta_edit_cursor);
 }
 
+void
+ARDOUR_UI::toggle_ShowTrackMeters()
+{
+	ActionManager::toggle_config_state ("options", "ShowTrackMeters", &Configuration::set_show_track_meters, &Configuration::get_show_track_meters);
+}
+
+void
+ARDOUR_UI::toggle_TapeMachineMode ()
+{
+	ActionManager::toggle_config_state ("options", "ToggleTapeMachineMode", &Configuration::set_tape_machine_mode, &Configuration::get_tape_machine_mode);
+}
+
+void
+ARDOUR_UI::toggle_use_narrow_ms()
+{
+	ActionManager::toggle_config_state ("options", "DefaultNarrowMS", &Configuration::set_default_narrow_ms, &Configuration::get_default_narrow_ms);
+}
+
+void
+ARDOUR_UI::toggle_rubberbanding_snaps_to_grid ()
+{
+	ActionManager::toggle_config_state ("options", "RubberbandingSnapsToGrid", &Configuration::set_rubberbanding_snaps_to_grid, &Configuration::get_rubberbanding_snaps_to_grid);
+}
+
+void
+ARDOUR_UI::toggle_auto_analyse_audio ()
+{
+	ActionManager::toggle_config_state ("options", "AutoAnalyseAudio", &Configuration::set_auto_analyse_audio, &Configuration::get_auto_analyse_audio);
+}
+
 void
 ARDOUR_UI::mtc_port_changed ()
 {
@@ -515,7 +610,7 @@ ARDOUR_UI::mtc_port_changed ()
 		positional_sync_strings.push_back (slave_source_to_string (MTC));
 	}
 	positional_sync_strings.push_back (slave_source_to_string (JACK));
-	
+
 	set_popdown_strings (sync_option_combo, positional_sync_strings);
 }
 
@@ -576,6 +671,19 @@ ARDOUR_UI::map_monitor_model ()
 	}
 }
 
+void
+ARDOUR_UI::map_denormal_protection ()
+{
+	Glib::RefPtr<Action> act = ActionManager::get_action ("options", X_("DenormalProtection"));
+	if (act) {
+		Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(act);
+
+		if (tact && !tact->get_active()) {
+			tact->set_active (Config->get_denormal_protection());
+		}
+	}
+}
+
 void
 ARDOUR_UI::map_denormal_model ()
 {
@@ -761,6 +869,20 @@ ARDOUR_UI::map_output_auto_connect ()
 	}
 }
 
+void
+ARDOUR_UI::map_only_copy_imported_files ()
+{
+	Glib::RefPtr<Action> act = ActionManager::get_action ("options", X_("OnlyCopyImportedFiles"));
+	if (act) {
+		Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(act);
+
+		if (tact && !tact->get_active()) {
+			tact->set_active (Config->get_only_copy_imported_files());
+		}
+	}
+
+}
+
 void
 ARDOUR_UI::map_meter_falloff ()
 {
@@ -943,6 +1065,7 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 
 	} else if (PARAM_IS ("send-mmc")) {
 
+
 		ActionManager::map_some_state ("options", "SendMMC", &Configuration::get_send_mmc);
 
 	} else if (PARAM_IS ("use-osc")) {
@@ -959,6 +1082,7 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		
 	} else if (PARAM_IS ("mmc-control")) {
 		ActionManager::map_some_state ("options", "UseMMC", &Configuration::get_mmc_control);
+
 	} else if (PARAM_IS ("midi-feedback")) {
 		ActionManager::map_some_state ("options", "SendMIDIfeedback", &Configuration::get_midi_feedback);
 	} else if (PARAM_IS ("do-not-record-plugins")) {
@@ -977,10 +1101,18 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		ActionManager::map_some_state ("Transport", "ToggleAutoReturn", &Configuration::get_auto_return);
 	} else if (PARAM_IS ("auto-input")) {
 		ActionManager::map_some_state ("Transport", "ToggleAutoInput", &Configuration::get_auto_input);
+	} else if (PARAM_IS ("tape-machine-mode")) {
+		ActionManager::map_some_state ("options", "ToggleTapeMachineMode", &Configuration::get_tape_machine_mode);
 	} else if (PARAM_IS ("punch-out")) {
 		ActionManager::map_some_state ("Transport", "TogglePunchOut", &Configuration::get_punch_out);
+		if (!Config->get_punch_out()) {
+			unset_dual_punch ();
+		}
 	} else if (PARAM_IS ("punch-in")) {
 		ActionManager::map_some_state ("Transport", "TogglePunchIn", &Configuration::get_punch_in);
+		if (!Config->get_punch_in()) {
+			unset_dual_punch ();
+		}
 	} else if (PARAM_IS ("clicking")) {
 		ActionManager::map_some_state ("Transport", "ToggleClick", &Configuration::get_clicking);
 	} else if (PARAM_IS ("jack-time-master")) {
@@ -995,12 +1127,18 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		ActionManager::map_some_state ("options",  "PeriodicSafetyBackups", &Configuration::get_periodic_safety_backups);
 	} else if (PARAM_IS ("stop-recording-on-xrun")) {
 		ActionManager::map_some_state ("options",  "StopRecordingOnXrun", &Configuration::get_stop_recording_on_xrun);
+	} else if (PARAM_IS ("create-xrun-marker")) {
+		ActionManager::map_some_state ("options",  "CreateXrunMarker", &Configuration::get_create_xrun_marker);
+	} else if (PARAM_IS ("sync-all-route-ordering")) {
+		ActionManager::map_some_state ("options",  "SyncEditorAndMixerTrackOrder", &Configuration::get_sync_all_route_ordering);
 	} else if (PARAM_IS ("stop-at-session-end")) {
 		ActionManager::map_some_state ("options",  "StopTransportAtEndOfSession", &Configuration::get_stop_at_session_end);
 	} else if (PARAM_IS ("monitoring-model")) {
 		map_monitor_model ();
 	} else if (PARAM_IS ("denormal-model")) {
 		map_denormal_model ();
+	} else if (PARAM_IS ("denormal-protection")) {
+		map_denormal_protection ();
 	} else if (PARAM_IS ("remote-model")) {
 		map_remote_model ();
 	} else if (PARAM_IS ("use-video-sync")) {
@@ -1062,7 +1200,17 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		ActionManager::map_some_state ("options",  "PrimaryClockDeltaEditCursor", &Configuration::get_primary_clock_delta_edit_cursor);
 	} else if (PARAM_IS ("secondary-clock-delta-edit-cursor")) {
 		ActionManager::map_some_state ("options",  "SecondaryClockDeltaEditCursor", &Configuration::get_secondary_clock_delta_edit_cursor);
-	} 
+	} else if (PARAM_IS ("only-copy-imported-files")) {
+		map_only_copy_imported_files ();
+	} else if (PARAM_IS ("show-track-meters")) {
+		ActionManager::map_some_state ("options",  "ShowTrackMeters", &Configuration::get_show_track_meters);
+		editor->toggle_meter_updating();
+	} else if (PARAM_IS ("default-narrow_ms")) {
+		ActionManager::map_some_state ("options",  "DefaultNarrowMS", &Configuration::get_default_narrow_ms);
+	} else if (PARAM_IS ("rubberbanding-snaps-to-grid")) {
+		ActionManager::map_some_state ("options", "RubberbandingSnapsToGrid", &Configuration::get_rubberbanding_snaps_to_grid);
+	}
+
 			   
 
 #undef PARAM_IS
