@@ -28,6 +28,7 @@
 #include <ardour/ardour.h>
 #include <ardour/session.h>
 #include <ardour/tempo.h>
+#include <ardour/profile.h>
 #include <sigc++/bind.h>
 
 #include "ardour_ui.h"
@@ -82,7 +83,8 @@ AudioClock::AudioClock (std::string clock_name, bool transient, std::string widg
 	key_entry_state = 0;
 	ops_menu = 0;
 	dragging = false;
-	
+	bbt_reference_time = -1;
+
 	if (with_info) {
 		frames_upper_info_label = manage (new Label);
 		frames_lower_info_label = manage (new Label);
@@ -500,15 +502,21 @@ AudioClock::set_frames (nframes_t when, bool force)
 			sprintf (buf, "%.3fK", rate/1000.0f);
 		}
 		
-		frames_upper_info_label->set_text (buf);
+		if (frames_upper_info_label->get_text() != buf) {
+			frames_upper_info_label->set_text (buf);
+		}
 		
 		float vid_pullup = Config->get_video_pullup();
 		
 		if (vid_pullup == 0.0) {
-			frames_lower_info_label->set_text(_("none"));
+			if (frames_lower_info_label->get_text () != _("none")) {
+				frames_lower_info_label->set_text(_("none"));
+			}
 		} else {
 			sprintf (buf, "%-6.4f", vid_pullup);
-			frames_lower_info_label->set_text (buf);
+			if (frames_lower_info_label->get_text() != buf) {
+				frames_lower_info_label->set_text (buf);
+			}
 		}
 	}
 }	
@@ -598,7 +606,9 @@ AudioClock::set_smpte (nframes_t when, bool force)
 			sprintf (buf, "%.2f", smpte_frames);
 		}
 		
-		smpte_upper_info_label->set_text (buf);
+		if (smpte_upper_info_label->get_text() != buf) {
+			smpte_upper_info_label->set_text (buf);
+		}
 		
 		if ((fabs(smpte_frames - 29.97) < 0.0001) || smpte_frames == 30) {
 			if (session->smpte_drop_frames()) {
@@ -611,7 +621,9 @@ AudioClock::set_smpte (nframes_t when, bool force)
 			buf[0] = '\0';
 		}
 		
-		smpte_lower_info_label->set_text (buf);
+		if (smpte_lower_info_label->get_text() != buf) {
+			smpte_lower_info_label->set_text (buf);
+		}
 	}
 }
 
@@ -622,6 +634,15 @@ AudioClock::set_bbt (nframes_t when, bool force)
 	BBT_Time bbt;
 
 	session->tempo_map().bbt_time (when, bbt);
+
+	/* handle a common case */
+
+	if (is_duration && when == 0) {
+		bbt.bars = 0;
+		bbt.beats = 0;
+
+	}
+
 	sprintf (buf, "%03" PRIu32, bbt.bars);
 	bars_label.set_text (buf);
 	sprintf (buf, "%02" PRIu32, bbt.beats);
@@ -630,11 +651,24 @@ AudioClock::set_bbt (nframes_t when, bool force)
 	ticks_label.set_text (buf);
 	
 	if (bbt_upper_info_label) {
-		TempoMap::Metric m (session->tempo_map().metric_at (when));
+		nframes64_t pos;
+
+		if (bbt_reference_time < 0) {
+			pos = when;
+		} else {
+			pos = bbt_reference_time;
+		}
+
+		TempoMap::Metric m (session->tempo_map().metric_at (pos));
+
 		sprintf (buf, "%-5.2f", m.tempo().beats_per_minute());
-		bbt_lower_info_label->set_text (buf);
+		if (bbt_lower_info_label->get_text() != buf) {
+			bbt_lower_info_label->set_text (buf);
+		}
 		sprintf (buf, "%g|%g", m.meter().beats_per_bar(), m.meter().note_divisor());
-		bbt_upper_info_label->set_text (buf);
+		if (bbt_upper_info_label->get_text() != buf) {
+			bbt_upper_info_label->set_text (buf);
+		}
 	}
 }
 
@@ -1883,7 +1917,9 @@ AudioClock::build_ops_menu ()
 	MenuList& ops_items = ops_menu->items();
 	ops_menu->set_name ("ArdourContextMenu");
 	
-	ops_items.push_back (MenuElem (_("Timecode"), bind (mem_fun(*this, &AudioClock::set_mode), SMPTE)));
+	if (!Profile->get_sae()) {
+		ops_items.push_back (MenuElem (_("Timecode"), bind (mem_fun(*this, &AudioClock::set_mode), SMPTE)));
+	}
 	ops_items.push_back (MenuElem (_("Bars:Beats"), bind (mem_fun(*this, &AudioClock::set_mode), BBT)));
 	ops_items.push_back (MenuElem (_("Minutes:Seconds"), bind (mem_fun(*this, &AudioClock::set_mode), MinSec)));
 	ops_items.push_back (MenuElem (_("Samples"), bind (mem_fun(*this, &AudioClock::set_mode), Frames)));
@@ -1978,3 +2014,9 @@ AudioClock::set_size_requests ()
 		
 	}
 }
+
+void
+AudioClock::set_bbt_reference (nframes64_t pos)
+{
+	bbt_reference_time = pos;
+}
