@@ -116,13 +116,35 @@ AudioRegionView::AudioRegionView (const AudioRegionView& other)
 	init (c, true);
 }
 
+AudioRegionView::AudioRegionView (const AudioRegionView& other, boost::shared_ptr<AudioRegion> other_region)
+	: RegionView (other, boost::shared_ptr<Region> (other_region))
+	, zero_line(0)
+	, fade_in_shape(0)
+	, fade_out_shape(0)
+	, fade_in_handle(0)
+	, fade_out_handle(0)
+	, gain_line(0)
+	, _amplitude_above_axis(1.0)
+	, _flags(0)
+	, fade_color(0)
+
+{
+	Gdk::Color c;
+	int r,g,b,a;
+
+	UINT_TO_RGBA (other.fill_color, &r, &g, &b, &a);
+	c.set_rgb_p (r/255.0, g/255.0, b/255.0);
+	
+	init (c, true);
+}
+
 void
 AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 {
 	// FIXME: Some redundancy here with RegionView::init.  Need to figure out
 	// where order is important and where it isn't...
 	
-	RegionView::init (basic_color, true);
+	RegionView::init (basic_color, wfd);
 	
 	XMLNode *node;
 
@@ -142,7 +164,7 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	}
 
 	compute_colors (basic_color);
-
+	
 	create_waves ();
 
 	fade_in_shape = new ArdourCanvas::Polygon (*group);
@@ -763,6 +785,7 @@ AudioRegionView::set_envelope_visible (bool yn)
 void
 AudioRegionView::create_waves ()
 {
+	// cerr << "AudioRegionView::create_waves() called on " << this << endl;//DEBUG
 	RouteTimeAxisView& atv (*(dynamic_cast<RouteTimeAxisView*>(&trackview))); // ick
 
 	if (!atv.get_diskstream()) {
@@ -770,6 +793,8 @@ AudioRegionView::create_waves ()
 	}
 
 	uint32_t nchans = atv.get_diskstream()->n_channels();
+
+	// cerr << "creating waves for " << _region->name() << " with wfd = " << wait_for_data << " and channels = " << nchans << endl;
 	
 	/* in tmp_waves, set up null pointers for each channel so the vector is allocated */
 	for (uint32_t n = 0; n < nchans; ++n) {
@@ -784,24 +809,30 @@ AudioRegionView::create_waves ()
 		
 		wave_caches.push_back (WaveView::create_cache ());
 
-		if (wait_for_data) {
+		// cerr << "\tchannel " << n << endl;
 
+		if (wait_for_data) {
 			if (audio_region()->source(n)->peaks_ready (bind (mem_fun(*this, &AudioRegionView::peaks_ready_handler), n), data_ready_connection)) {
+				// cerr << "\tData is ready\n";
 				create_one_wave (n, true);
 			} else {
+				// cerr << "\tdata is not ready\n";
 				// we'll get a PeaksReady signal from the source in the future
 				// and will call create_one_wave(n) then.
 			}
-
+			
 		} else {
+			// cerr << "\tdon't delay, display today!\n";
 			create_one_wave (n, true);
 		}
+
 	}
 }
 
 void
 AudioRegionView::create_one_wave (uint32_t which, bool direct)
 {
+	//cerr << "AudioRegionView::create_one_wave() called which: " << which << " this: " << this << endl;//DEBUG
 	RouteTimeAxisView& atv (*(dynamic_cast<RouteTimeAxisView*>(&trackview))); // ick
 	uint32_t nchans = atv.get_diskstream()->n_channels();
 	uint32_t n;
@@ -831,6 +862,7 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 	wave->property_samples_per_unit() =  samples_per_unit;
 	wave->property_amplitude_above_axis() =  _amplitude_above_axis;
 	wave->property_wave_color() = _region->muted() ? UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA) : ARDOUR_UI::config()->canvasvar_WaveForm.get();
+	wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
 	wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_WaveFormClip.get();
 	wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_ZeroLine.get();
 	wave->property_region_start() = _region->start();
@@ -885,6 +917,7 @@ void
 AudioRegionView::peaks_ready_handler (uint32_t which)
 {
 	Gtkmm2ext::UI::instance()->call_slot (bind (mem_fun(*this, &AudioRegionView::create_one_wave), which, false));
+	// cerr << "AudioRegionView::peaks_ready_handler() called on " << which << " this: " << this << endl;
 }
 
 void
@@ -1078,6 +1111,7 @@ AudioRegionView::add_ghost (AutomationTimeAxisView& atv)
 		wave->property_samples_per_unit() =  samples_per_unit;
 		wave->property_amplitude_above_axis() =  _amplitude_above_axis;
 		wave->property_wave_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWave.get();
+		wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWave.get();
 		wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWaveClip.get();
 		wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_GhostTrackZeroLine.get();
 		wave->property_region_start() = _region->start();
@@ -1181,14 +1215,15 @@ AudioRegionView::set_frame_color ()
 	uint32_t r,g,b,a;
 	
 	if (_selected && should_show_selection) {
-		frame->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
+		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_SelectedFrameBase.get(), &r, &g, &b, &a);
+		frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
 
-		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
 		for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
 			if (_region->muted()) {
-				(*w)->property_wave_color() = RGBA_TO_UINT(r, g, b, MUTED_ALPHA);
+				(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_SelectedWaveForm.get(), MUTED_ALPHA);
 			} else {
-				(*w)->property_wave_color() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);// Lets still use the theme's opacity value if Opaque is not set
+				(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_SelectedWaveForm.get();
+				(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_SelectedWaveFormFill.get();
 			}
 		}
 	} else {
@@ -1200,6 +1235,7 @@ AudioRegionView::set_frame_color ()
 				(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
 			} else {
 				(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
+				(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
 			}
 		}
 	}
