@@ -55,13 +55,16 @@ class AudioRegionView : public RegionView
 			 Gdk::Color& basic_color);
 
         AudioRegionView (const AudioRegionView& other);
+        AudioRegionView (const AudioRegionView& other, boost::shared_ptr<ARDOUR::AudioRegion>);
 
 	~AudioRegionView ();
 	
-	virtual void init (Gdk::Color& base_color, bool wait_for_data = false);
+	virtual void init (Gdk::Color& base_color, bool wait_for_data);
 	
 	boost::shared_ptr<ARDOUR::AudioRegion> audio_region() const;
 	
+	void create_waves ();
+
 	void set_height (double);
 	void set_samples_per_unit (double);
 	
@@ -72,8 +75,8 @@ class AudioRegionView : public RegionView
 	
 	void set_envelope_visible (bool);
 	void set_waveform_visible (bool yn);
-	void set_waveform_shape (WaveformShape);
-	void set_waveform_scale (WaveformScale);
+	void set_waveform_shape (Editing::WaveformShape);
+	void set_waveform_scale (Editing::WaveformScale);
 	
 	bool waveform_rectified() const { return _flags & WaveformRectified; }
 	bool waveform_logscaled() const { return _flags & WaveformLogScaled; }
@@ -132,7 +135,6 @@ class AudioRegionView : public RegionView
     void region_muted ();
     void region_scale_amplitude_changed ();
 
-    void create_waves ();
     void create_one_wave (uint32_t, bool);
     void manage_zero_line ();
     void peaks_ready_handler (uint32_t);
