@@ -59,6 +59,7 @@
 #include "editor_items.h"
 #include "region_selection.h"
 #include "canvas.h"
+#include "time_axis_view.h"
 #include "draginfo.h"
 
 namespace Gtkmm2ext {
@@ -105,6 +106,7 @@ class StreamView;
 class AudioStreamView;
 class ControlPoint;
 class SoundFileOmega;
+class RhythmFerret;
 #ifdef FFT_ANALYSIS
 class AnalysisWindow;
 #endif
@@ -128,6 +130,8 @@ class Editor : public PublicEditor
 
 	void             connect_to_session (ARDOUR::Session *);
 	ARDOUR::Session* current_session() const { return session; }
+	void             first_idle ();
+	virtual bool have_idled() const { return _have_idled; }
 
 	nframes_t leftmost_position() const { return leftmost_frame; }
 	nframes_t current_page_frames() const {
@@ -155,6 +159,7 @@ class Editor : public PublicEditor
 	void connect_to_image_compositor() ;
 	void scroll_timeaxis_to_imageframe_item(const TimeAxisViewItem* item) ;
 	TimeAxisView* get_named_time_axis(const std::string & name) ;
+	void foreach_time_axis_view (sigc::slot<void,TimeAxisView&>);
 
 	void consider_auditioning (boost::shared_ptr<ARDOUR::Region>);
 	void hide_a_region (boost::shared_ptr<ARDOUR::Region>);
@@ -169,6 +174,8 @@ class Editor : public PublicEditor
 	void set_show_waveforms (bool yn);
 	bool show_waveforms() const { return _show_waveforms; }
 
+	void set_waveform_scale (Editing::WaveformScale);
+
 	void set_show_waveforms_recording (bool yn);
 	bool show_waveforms_recording() const { return _show_waveforms_recording; }
 	
@@ -177,6 +184,8 @@ class Editor : public PublicEditor
 	void new_region_from_selection ();
 	void separate_regions_between (const TimeSelection&);
 	void separate_region_from_selection ();
+	void separate_region_from_punch ();
+	void separate_region_from_loop ();
 	void separate_regions_using_location (ARDOUR::Location&);
 	void toggle_playback (bool with_abort);
 	void transition_to_rolling (bool forward);
@@ -211,14 +220,14 @@ class Editor : public PublicEditor
 		*/
 
 		if (pixel >= 0) {
-			return (nframes_t) rint (pixel * frames_per_unit * GNOME_CANVAS(track_canvas.gobj())->pixels_per_unit);
+			return (nframes_t) rint (pixel * frames_per_unit * GNOME_CANVAS(track_canvas->gobj())->pixels_per_unit);
 		} else {
 			return 0;
 		}
 	}
 
 	gulong frame_to_pixel (nframes64_t frame) const {
-		return (gulong) rint ((frame / (frames_per_unit *  GNOME_CANVAS(track_canvas.gobj())->pixels_per_unit)));
+		return (gulong) rint ((frame / (frames_per_unit *  GNOME_CANVAS(track_canvas->gobj())->pixels_per_unit)));
 	}
 
 	void flush_canvas ();
@@ -286,8 +295,8 @@ class Editor : public PublicEditor
 
 	/* nudge is initiated by transport controls owned by ARDOUR_UI */
 
-	void nudge_forward (bool next);
-	void nudge_backward (bool next);
+	void nudge_forward (bool next, bool force_playhead);
+	void nudge_backward (bool next, bool force_playhead);
 
 	/* nudge initiated from context menu */
 
@@ -361,6 +370,8 @@ class Editor : public PublicEditor
 
 	void toggle_meter_updating();
 
+	void show_rhythm_ferret();
+
   protected:
 	void map_transport_state ();
 	void map_position_change (nframes_t);
@@ -453,7 +464,7 @@ class Editor : public PublicEditor
 
 	void hide_marker (ArdourCanvas::Item*, GdkEvent*);
 	void clear_marker_display ();
-	void mouse_add_new_marker (nframes_t where, bool is_cd=false);
+	void mouse_add_new_marker (nframes_t where, bool is_cd=false, bool is_xrun=false);
 	void update_cd_marker_display ();
 	void ensure_cd_marker_updated (LocationMarkers * lam, ARDOUR::Location * location);
 
@@ -529,9 +540,11 @@ class Editor : public PublicEditor
 	Gtk::VBox           vpacker;
 
 	Gdk::Cursor*          current_canvas_cursor;
+	void set_canvas_cursor ();
+	Gdk::Cursor* which_grabber_cursor ();
 
-	ArdourCanvas::CanvasAA track_canvas;
-	ArdourCanvas::CanvasAA time_canvas;
+	ArdourCanvas::Canvas* track_canvas;
+	ArdourCanvas::Canvas* time_canvas;
 
 	ArdourCanvas::Text* first_action_message;
 	ArdourCanvas::Text* verbose_canvas_cursor;
@@ -565,7 +578,7 @@ class Editor : public PublicEditor
 	ArdourCanvas::Group*      transport_marker_group;
 	ArdourCanvas::Group*      cd_marker_group;
 	
-	enum {
+	enum RulerType {
 		ruler_metric_smpte = 0,
 		ruler_metric_bbt = 1,
 		ruler_metric_frames = 2,
@@ -580,7 +593,16 @@ class Editor : public PublicEditor
 	};
 
 	static GtkCustomMetric ruler_metrics[4];
-	bool                   ruler_shown[10];
+	Glib::RefPtr<Gtk::ToggleAction> ruler_timecode_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_bbt_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_samples_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_minsec_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_tempo_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_meter_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_marker_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_range_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_loop_punch_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_cd_marker_action;
 	bool                   no_ruler_shown_update;
 	
 	gint ruler_button_press (GdkEventButton*);
@@ -597,6 +619,8 @@ class Editor : public PublicEditor
 	void update_tempo_based_rulers (); 
 	void popup_ruler_menu (nframes_t where = 0, ItemType type = RegionItem);
 	void update_ruler_visibility ();
+	void set_ruler_visible (RulerType, bool);
+	void toggle_ruler_visibility (RulerType rt);
 	void ruler_toggled (int);
 	gint ruler_label_button_release (GdkEventButton*);
 	void store_ruler_visibility ();
@@ -891,6 +915,7 @@ class Editor : public PublicEditor
 	static Gdk::Cursor* trimmer_cursor;
 	static Gdk::Cursor* selector_cursor;
 	static Gdk::Cursor* grabber_cursor;
+	static Gdk::Cursor* grabber_edit_point_cursor;
 	static Gdk::Cursor* zoom_cursor;
 	static Gdk::Cursor* time_fx_cursor;
 	static Gdk::Cursor* fader_cursor;
@@ -927,8 +952,10 @@ class Editor : public PublicEditor
 	int ensure_cursor (nframes_t* pos);
 
 	void handle_new_audio_region (boost::weak_ptr<ARDOUR::AudioRegion>);
+	void handle_new_audio_regions (vector<boost::weak_ptr<ARDOUR::AudioRegion> >& );
 	void handle_audio_region_removed (boost::weak_ptr<ARDOUR::AudioRegion>);
 	void add_audio_region_to_region_display (boost::shared_ptr<ARDOUR::AudioRegion>);
+	void add_audio_regions_to_region_display (std::vector<boost::weak_ptr<ARDOUR::AudioRegion> > & );
 	void region_hidden (boost::shared_ptr<ARDOUR::Region>);
 	void redisplay_regions ();
 	void insert_into_tmp_audio_regionlist(boost::shared_ptr<ARDOUR::AudioRegion>);
@@ -937,7 +964,7 @@ class Editor : public PublicEditor
 
 	void cut_copy (Editing::CutCopyOp);
 	void cut_copy_points (Editing::CutCopyOp);
-	void cut_copy_regions (Editing::CutCopyOp);
+	void cut_copy_regions (Editing::CutCopyOp, RegionSelection&);
 	void cut_copy_ranges (Editing::CutCopyOp);
 
 	void mouse_paste ();
@@ -949,6 +976,7 @@ class Editor : public PublicEditor
 	void toggle_region_mute ();
 	void toggle_region_lock ();
 	void toggle_region_opaque ();
+	void set_region_lock_style (ARDOUR::Region::PositionLockStyle);
 	void raise_region ();
 	void raise_region_to_top ();
 	void lower_region ();
@@ -956,6 +984,8 @@ class Editor : public PublicEditor
 	void split_region ();
 	void split_region_at (nframes_t);
 	void split_regions_at (nframes_t, RegionSelection&);
+	void split_region_at_transients ();
+	void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&, bool can_ferret);
 	void crop_region_to_selection ();
 	void crop_region_to (nframes_t start, nframes_t end);
 	void set_sync_point (nframes64_t, const RegionSelection&);
@@ -980,6 +1010,17 @@ class Editor : public PublicEditor
 	void reverse_region ();
 	void normalize_region ();
 	void denormalize_region ();
+	void adjust_region_scale_amplitude (bool up);
+
+	void do_insert_time ();
+	void insert_time (nframes64_t pos, nframes64_t distance, Editing::InsertTimeOption opt, bool ignore_music_glue);
+
+	void tab_to_transient (bool forward);
+
+	void use_region_as_bar ();
+	void use_range_as_bar ();
+
+	void define_one_bar (nframes64_t start, nframes64_t end);
 
 	void audition_region_from_region_list ();
 	void hide_region_from_region_list ();
@@ -1031,19 +1072,24 @@ class Editor : public PublicEditor
 
 	void add_external_audio_action (Editing::ImportMode);
 	void external_audio_dialog ();
+
+	int  check_whether_and_how_to_import(string, bool all_or_nothing = true);
 	bool check_multichannel_status (const std::vector<Glib::ustring>& paths);
 
 	SoundFileOmega* sfbrowser;
 	
 	void bring_in_external_audio (Editing::ImportMode mode,  nframes64_t& pos);
+
+	void _do_import (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, nframes64_t&);
 	void do_import (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, nframes64_t&);
+	bool idle_do_import (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, nframes64_t&);
 
 	void _do_embed (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode,  nframes64_t&);
 	void do_embed (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode,  nframes64_t&);
 	bool idle_do_embed (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode,  nframes64_t&);
 
 	int  import_sndfiles (vector<Glib::ustring> paths, Editing::ImportMode mode,  ARDOUR::SrcQuality, nframes64_t& pos,
-			      int target_regions, int target_tracks, boost::shared_ptr<ARDOUR::AudioTrack>&);
+			      int target_regions, int target_tracks, boost::shared_ptr<ARDOUR::AudioTrack>&, bool);
 	int  embed_sndfiles (vector<Glib::ustring> paths, bool multiple_files, bool& check_sample_rate, Editing::ImportMode mode, 
 			     nframes64_t& pos, int target_regions, int target_tracks, boost::shared_ptr<ARDOUR::AudioTrack>&);
 
@@ -1090,6 +1136,8 @@ class Editor : public PublicEditor
 	void center_edit_point ();
 	void edit_cursor_backward ();
 	void edit_cursor_forward ();
+	void playhead_forward_to_grid ();
+	void playhead_backward_to_grid ();
 	void playhead_backward ();
 	void playhead_forward ();
 	void scroll_playhead (bool forward);
@@ -1149,6 +1197,7 @@ class Editor : public PublicEditor
 	int scrubbing_direction;
 	int scrub_reversals;
 	int scrub_reverse_distance;
+	void scrub ();
 
 	void keyboard_selection_begin ();
 	void keyboard_selection_finish (bool add);
@@ -1173,6 +1222,8 @@ class Editor : public PublicEditor
 	void start_grab (GdkEvent*, Gdk::Cursor* cursor = 0);
 	bool end_grab (ArdourCanvas::Item*, GdkEvent*);
 	void swap_grab (ArdourCanvas::Item*, Gdk::Cursor* cursor, uint32_t time);
+	void break_drag ();
+	void finalize_drag ();
 
 	Gtk::Menu fade_context_menu;
 	void popup_fade_context_menu (int, int, ArdourCanvas::Item*, ItemType);
@@ -1245,6 +1296,8 @@ class Editor : public PublicEditor
 
 	void show_verbose_time_cursor (nframes_t frame, double offset = 0, double xpos=-1, double ypos=-1);
 	void show_verbose_duration_cursor (nframes_t start, nframes_t end, double offset = 0, double xpos=-1, double ypos=-1);
+	double clamp_verbose_cursor_x (double);
+	double clamp_verbose_cursor_y (double);
 
 	/* Canvas event handlers */
 
@@ -1258,6 +1311,14 @@ class Editor : public PublicEditor
 	bool canvas_fade_in_handle_event (GdkEvent* event,ArdourCanvas::Item*, AudioRegionView*);
 	bool canvas_fade_out_event (GdkEvent* event,ArdourCanvas::Item*, AudioRegionView*);
 	bool canvas_fade_out_handle_event (GdkEvent* event,ArdourCanvas::Item*, AudioRegionView*);
+	
+
+	// These variables are used to detect a feedback loop and break it to avoid a gui hang
+private:
+	ArdourCanvas::Item *last_item_entered;
+	int last_item_entered_n;
+public:
+
 	bool canvas_region_view_event (GdkEvent* event,ArdourCanvas::Item*, RegionView*);
 	bool canvas_region_view_name_highlight_event (GdkEvent* event,ArdourCanvas::Item*, RegionView*);
 	bool canvas_region_view_name_event (GdkEvent* event,ArdourCanvas::Item*, RegionView*);
@@ -1494,6 +1555,7 @@ class Editor : public PublicEditor
 	void time_selection_changed ();
 	void track_selection_changed ();
 	void region_selection_changed ();
+	void sensitize_the_right_region_actions (bool have_selected_regions);
 	void point_selection_changed ();
 	void marker_selection_changed ();
 
@@ -1526,6 +1588,7 @@ class Editor : public PublicEditor
 	void drag_range_markerbar_op (ArdourCanvas::Item* item, GdkEvent* event);
 	void end_range_markerbar_op (ArdourCanvas::Item* item, GdkEvent* event);
 
+	ArdourCanvas::SimpleRect*  cd_marker_bar_drag_rect;
 	ArdourCanvas::SimpleRect*  range_bar_drag_rect;
 	ArdourCanvas::SimpleRect*  transport_bar_drag_rect;
 	ArdourCanvas::Line*        marker_drag_line;
@@ -1670,15 +1733,19 @@ class Editor : public PublicEditor
 
 	bool autoscroll_active;
 	int autoscroll_timeout_tag;
-	int autoscroll_direction;
+	int autoscroll_x;
+	int autoscroll_y;
+	int last_autoscroll_x;
+	int last_autoscroll_y;
 	uint32_t autoscroll_cnt;
-	nframes_t autoscroll_distance;
+	nframes_t autoscroll_x_distance;
+	double autoscroll_y_distance;
      
 	static gint _autoscroll_canvas (void *);
 	bool autoscroll_canvas ();
-	void start_canvas_autoscroll (int direction);
+	void start_canvas_autoscroll (int x, int y);
 	void stop_canvas_autoscroll ();
-	void maybe_autoscroll (GdkEvent*);
+	void maybe_autoscroll (GdkEventMotion*);
 
 	/* trimming */
 	enum TrimOp {
@@ -1960,6 +2027,9 @@ class Editor : public PublicEditor
 	AudioClock       nudge_clock;
 
 	nframes_t get_nudge_distance (nframes_t pos, nframes_t& next);
+
+	bool nudge_forward_release (GdkEventButton*);
+	bool nudge_backward_release (GdkEventButton*);
 	
 	/* audio filters */
 
@@ -1981,27 +2051,18 @@ class Editor : public PublicEditor
 	TimeAxisView* entered_track;
 	RegionView*   entered_regionview;
 
-	class ExclusiveRegionSelection {
-	  public:
-		ExclusiveRegionSelection (Editor&, RegionView*);
-		~ExclusiveRegionSelection ();
 
-	  private:
-		Editor& editor;
-		RegionView* regionview;
-		bool remove;
-	};
-
-	void ensure_entered_region_selected (bool op_acts_on_objects = false);
 	void ensure_entered_track_selected (bool op_acts_on_objects = false);
 	bool clear_entered_track;
-	gint left_track_canvas (GdkEventCrossing*);
+	bool left_track_canvas (GdkEventCrossing*);
+	bool entered_track_canvas (GdkEventCrossing*);
 	void set_entered_track (TimeAxisView*);
 	void set_entered_regionview (RegionView*);
 	gint left_automation_track ();
 
 	bool _new_regionviews_show_envelope;
 
+	void reset_canvas_action_sensitivity (bool);
 	void toggle_gain_envelope_visibility ();
 	void toggle_gain_envelope_active ();
 	void reset_region_gain_envelopes ();
@@ -2023,7 +2084,7 @@ class Editor : public PublicEditor
 
 	Gtk::ComboBoxText edit_point_selector;
 
-	void set_edit_point_preference (Editing::EditPoint ep);
+	void set_edit_point_preference (Editing::EditPoint ep, bool force = false);
 	void cycle_edit_point (bool with_marker);
 	void set_edit_point ();
 	void edit_point_selection_done ();
@@ -2037,12 +2098,10 @@ class Editor : public PublicEditor
 
 	bool get_edit_op_range (nframes64_t& start, nframes64_t& end) const;
 
-	RegionSelection get_regions_at (nframes64_t where, const TrackSelection& ts) const;
-	RegionSelection get_regions_after (nframes64_t where, const TrackSelection& ts) const;
+	void get_regions_at (RegionSelection&, nframes64_t where, const TrackSelection& ts) const;
+	void get_regions_after (RegionSelection&, nframes64_t where, const TrackSelection& ts) const;
 	
-	RegionSelection tmp_regions;
-	
-	RegionSelection& get_regions_for_action ();
+	void get_regions_for_action (RegionSelection&, bool allowed_entered_regionview = false);
 
 	sigc::connection fast_screen_update_connection;
 	gint start_updating ();
@@ -2052,6 +2111,24 @@ class Editor : public PublicEditor
 
 	void select_next_route ();
 	void select_prev_route ();
+
+	void snap_to_internal (nframes64_t& first, int32_t direction = 0, bool for_mark = false);
+
+	RhythmFerret* rhythm_ferret;
+
+	void set_track_height (TimeAxisView::TrackHeight h);
+	void set_track_height_largest ();
+	void set_track_height_large ();
+	void set_track_height_larger ();
+	void set_track_height_normal ();
+	void set_track_height_smaller ();
+	void set_track_height_small ();
+
+	void remove_tracks ();
+	void toggle_tracks_active ();
+	void waveform_scale_chosen (Editing::WaveformScale);
+
+	bool _have_idled;
 };
 
 #endif /* __ardour_editor_h__ */
