@@ -22,6 +22,7 @@
 #include <sys/time.h>
 #include <errno.h>
 #include <unistd.h>
+#include <algorithm>
 
 #include <sndfile.h>
 
@@ -42,6 +43,7 @@
 #include <ardour/audiofilesource.h>
 #include <ardour/region_factory.h>
 #include <ardour/source_factory.h>
+#include <ardour/session.h>
 #include <pbd/memento_command.h>
 
 #include "ardour_ui.h"
@@ -114,60 +116,139 @@ Editor::external_audio_dialog ()
 
 	sfbrowser->show_all ();
 
-  again:
-	int response = sfbrowser->run ();
 
-	switch (response) {
-	case RESPONSE_APPLY:
-		// leave the dialog open
-		break;
+	bool keepRunning;
 
-	case RESPONSE_OK:
-		sfbrowser->hide ();
-		break;
+	do {
+		keepRunning = false;
 
-	default:
-		// cancel from the browser - we are done
-		sfbrowser->hide ();
-		return;
+		int response = sfbrowser->run ();
+
+		switch (response) {
+			case RESPONSE_APPLY:
+				// leave the dialog open
+				break;
+
+			case RESPONSE_OK:
+				sfbrowser->hide ();
+				break;
+
+			default:
+				// cancel from the browser - we are done
+				sfbrowser->hide ();
+				return;
+		}
+
+		/* lets do it */
+
+		paths = sfbrowser->get_paths ();
+
+		ImportPosition pos = sfbrowser->get_position ();
+		ImportMode mode = sfbrowser->get_mode ();
+		ImportDisposition chns = sfbrowser->get_channel_disposition ();
+		nframes64_t where;
+
+		switch (pos) {
+			case ImportAtEditPoint:
+				where = get_preferred_edit_position ();
+				break;
+			case ImportAtTimestamp:
+				where = -1;
+				break;
+			case ImportAtPlayhead:
+				where = playhead_cursor->current_frame;
+				break;
+			case ImportAtStart:
+				where = session->current_start_frame();
+				break;
+		}
+
+		SrcQuality quality = sfbrowser->get_src_quality();
+
+
+		if (sfbrowser->copy_files_btn.get_active()) {
+			do_import (paths, chns, mode, quality, where);
+		} else {
+			do_embed (paths, chns, mode, where);
+		}
+
+		if (response == RESPONSE_APPLY) {
+			sfbrowser->clear_selection ();
+			keepRunning = true;
+		}
+
+	} while (keepRunning);
+}
+
+typedef std::map<PBD::ID,boost::shared_ptr<AudioSource> > AudioSourceList;
+
+/**
+ * Updating is still disabled, see note in libs/ardour/import.cc Session::import_audiofiles()
+ *
+ * all_or_nothing:
+ *   true  = show "Update", "Import" and "Skip"
+ *   false = show "Import", and "Cancel"
+ *
+ * Returns:
+ *     0  To update an existing source of the same name
+ *     1  To import/embed the file normally (make sure the new name will be unique)
+ *     2  If the user wants to skip this file
+ **/
+int
+Editor::check_whether_and_how_to_import(string path, bool all_or_nothing)
+{
+	string wave_name (Glib::path_get_basename(path));
+
+	AudioSourceList all_sources = session->get_audio_sources();
+	bool wave_name_exists = false;
+
+	for (AudioSourceList::iterator i = all_sources.begin(); i != all_sources.end(); ++i) {
+		boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(i->second);
+
+		string tmp (Glib::path_get_basename (afs->path()));
+
+		if (tmp == wave_name) {
+			wave_name_exists = true;
+			break;
+		}
 	}
 
-	/* lets do it */
-	
-	paths = sfbrowser->get_paths ();
+	int function = 1;
 
-	ImportPosition pos = sfbrowser->get_position ();
-	ImportMode mode = sfbrowser->get_mode ();
-	ImportDisposition chns = sfbrowser->get_channel_disposition ();
-	nframes64_t where;
 
-	switch (pos) {
-	case ImportAtEditPoint:
-		where = get_preferred_edit_position ();
-		break;
-	case ImportAtTimestamp:
-		where = -1;
-		break;
-	case ImportAtPlayhead:
-		where = playhead_cursor->current_frame;
-		break;
-	case ImportAtStart:
-		where = session->current_start_frame();
-		break;
+	if (wave_name_exists) {
+		string message;
+		if (all_or_nothing) {
+			// updating is still disabled
+			//message = string_compose(_("The session already contains a source file named %1. Do you want to update that file (and thus all regions using the file) or import this file as a new file?"),wave_name);
+			message = string_compose(_("The session already contains a source file named %1. This file will be imported as a new file, please confirm."),wave_name);
+		} else {
+			message = string_compose(_("A source file %1 already exists. This operation will not update that source but import the file %2 as a new source, please confirm."), wave_name, wave_name);
+
+		}
+		MessageDialog dialog(message, false,Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
+
+		if (all_or_nothing) {
+			// disabled
+			//dialog.add_button("Update", 0);
+			dialog.add_button("Import", 1);
+			dialog.add_button("Skip",   2);
+		} else {
+			dialog.add_button("Import", 1);
+			dialog.add_button("Cancel", 2);
+		}
+		
+		
+		//dialog.add_button("Skip all", 4); // All or rest?
+
+		dialog.show();
+
+		function = dialog.run ();
+
+		dialog.hide();
 	}
 
-	SrcQuality quality = sfbrowser->get_src_quality();
-
-	if (sfbrowser->copy_files_btn.get_active()) {
-		do_import (paths, chns, mode, quality, where);
-	} else {
-		do_embed (paths, chns, mode, where);
-	}
-
-	if (response == RESPONSE_APPLY) {
-		sfbrowser->clear_selection ();
-		goto again;
-	}
+	return function;
 }
 
 boost::shared_ptr<AudioTrack>
@@ -200,84 +281,128 @@ Editor::get_nth_selected_audio_track (int nth) const
 	return atv->audio_track();
 }	
 
+bool
+Editor::idle_do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mode, SrcQuality quality, nframes64_t& pos)
+{
+	_do_import (paths, chns, mode, quality, pos);
+	return false;
+}
+
 void
 Editor::do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mode, SrcQuality quality, nframes64_t& pos)
+{
+#ifdef GTKOSX
+	Glib::signal_idle().connect (bind (mem_fun (*this, &Editor::idle_do_import), paths, chns, mode, quality, pos));
+#else
+	_do_import (paths, chns, mode, quality, pos);
+#endif
+}
+
+void
+Editor::_do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mode, SrcQuality quality, nframes64_t& pos)
 {
 	boost::shared_ptr<AudioTrack> track;
 	vector<ustring> to_import;
-	bool ok = false;
+	bool ok = true;
 	int nth = 0;
 
 	if (interthread_progress_window == 0) {
 		build_interthread_progress_window ();
 	}
 
-	switch (chns) {
-	case Editing::ImportDistinctFiles:
-		for (vector<ustring>::iterator a = paths.begin(); a != paths.end(); ++a) {
-
-			to_import.clear ();
-			to_import.push_back (*a);
-
-			if (mode == Editing::ImportToTrack) {
-				track = get_nth_selected_audio_track (nth++);
-			}
-			
-			if (import_sndfiles (to_import, mode, quality, pos, 1, -1, track)) {
-				goto out;
-			}
-
-		}
-		break;
-
-	case Editing::ImportDistinctChannels:
-		for (vector<ustring>::iterator a = paths.begin(); a != paths.end(); ++a) {
-
-			to_import.clear ();
-			to_import.push_back (*a);
-
-			if (import_sndfiles (to_import, mode, quality, pos, -1, -1, track)) {
-				goto out;
-			}
-
-		}
-		break;
-
-	case Editing::ImportMergeFiles:
+	if (chns == Editing::ImportMergeFiles) {
 		/* create 1 region from all paths, add to 1 track,
 		   ignore "track"
 		*/
-		if (import_sndfiles (paths, mode, quality, pos, 1, 1, track)) {
-			goto out;
+		bool cancel = false;
+		for (vector<ustring>::iterator a = paths.begin(); a != paths.end() && ok; ++a) {
+			int check = check_whether_and_how_to_import(*a, false);
+			if (check == 2) {
+				cancel = true;
+				break;
+			}
 		}
-		break;
 
-	case Editing::ImportSerializeFiles:
-		for (vector<ustring>::iterator a = paths.begin(); a != paths.end(); ++a) {
+		if (!cancel) {
+			if (import_sndfiles (paths, mode, quality, pos, 1, 1, track, false)) {
+				ok = false;
+			}
+		}
 
-			to_import.clear ();
-			to_import.push_back (*a);
-			
-			/* create 1 region from this path, add to 1 track,
-			   reuse "track" across paths
-			*/
+	} else {
+		bool replace;
 
-			if (import_sndfiles (to_import, mode, quality, pos, 1, 1, track)) {
-				goto out;
+		for (vector<ustring>::iterator a = paths.begin(); a != paths.end() && ok; ++a) {
+
+			int check = check_whether_and_how_to_import(*a, true);
+
+			if (check == 2 ) { 
+				// skip
+				continue;
 			}
 
+			if (check == 0) {
+				fatal << "Updating existing sources should be disabled!" << endl;
+				replace = true;
+			} else if (check == 1) {
+				replace = false;
+			}
+			
+
+			switch (chns) {
+				case Editing::ImportDistinctFiles:
+
+					to_import.clear ();
+					to_import.push_back (*a);
+
+					if (mode == Editing::ImportToTrack) {
+						track = get_nth_selected_audio_track (nth++);
+					}
+
+					if (import_sndfiles (to_import, mode, quality, pos, 1, -1, track, replace)) {
+						ok = false;
+					}
+
+					break;
+
+				case Editing::ImportDistinctChannels:
+
+					to_import.clear ();
+					to_import.push_back (*a);
+
+					if (import_sndfiles (to_import, mode, quality, pos, -1, -1, track, replace)) {
+						ok = false;
+					}
+
+					break;
+
+				case Editing::ImportSerializeFiles:
+
+					to_import.clear ();
+					to_import.push_back (*a);
+
+					/* create 1 region from this path, add to 1 track,
+					   reuse "track" across paths
+					   */
+
+					if (import_sndfiles (to_import, mode, quality, pos, 1, 1, track, replace)) {
+						ok = false;
+					}
+
+					break;
+
+				case Editing::ImportMergeFiles:
+					// Not entered
+					break;
+			}
 		}
-		break;
-	}
 
-	ok = true;
-	
-  out:	
-	if (ok) {
-		session->save_state ("");
-	}
+		if (ok) {
+			session->save_state ("");
+		}
 
-	interthread_progress_window->hide_all ();
+		interthread_progress_window->hide_all ();
+	}
 }
 
 bool
@@ -365,13 +490,12 @@ Editor::_do_embed (vector<ustring> paths, ImportDisposition chns, ImportMode mod
 
 int
 Editor::import_sndfiles (vector<ustring> paths, ImportMode mode, SrcQuality quality, nframes64_t& pos, 
-			 int target_regions, int target_tracks, boost::shared_ptr<AudioTrack>& track)
+			 int target_regions, int target_tracks, boost::shared_ptr<AudioTrack>& track, bool replace)
 {
 	WindowTitle title = string_compose (_("importing %1"), paths.front());
 
 	interthread_progress_window->set_title (title.get_string());
 	interthread_progress_window->set_position (Gtk::WIN_POS_MOUSE);
-	interthread_progress_window->show_all ();
 	interthread_progress_bar.set_fraction (0.0f);
 	interthread_cancel_label.set_text (_("Cancel Import"));
 	current_interthread_info = &import_status;
@@ -382,11 +506,12 @@ Editor::import_sndfiles (vector<ustring> paths, ImportMode mode, SrcQuality qual
 	import_status.freeze = false;
 	import_status.done = 0.0;
 	import_status.quality = quality;
+	import_status.replace_existing_source = replace;
 
 	interthread_progress_connection = Glib::signal_timeout().connect 
-		(bind (mem_fun(*this, &Editor::import_progress_timeout), (gpointer) 0), 100);
+		(bind (mem_fun(*this, &Editor::import_progress_timeout), (gpointer) 0), 500);
 	
-	track_canvas.get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
+	track_canvas->get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
 	ARDOUR_UI::instance()->flush_pending ();
 
 	/* start import thread for this spec. this will ultimately call Session::import_audiofile()
@@ -418,7 +543,7 @@ Editor::import_sndfiles (vector<ustring> paths, ImportMode mode, SrcQuality qual
 	}
 
   out:
-	track_canvas.get_window()->set_cursor (*current_canvas_cursor);
+	track_canvas->get_window()->set_cursor (*current_canvas_cursor);
 	return 0;
 }
 
@@ -433,7 +558,7 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 	SoundFileInfo finfo;
 	int ret = 0;
 
-	track_canvas.get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
+	track_canvas->get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
 	ARDOUR_UI::instance()->flush_pending ();
 
 	for (vector<Glib::ustring>::iterator p = paths.begin(); p != paths.end(); ++p) {
@@ -534,7 +659,7 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 			}
 		}
 		
-		track_canvas.get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
+		track_canvas->get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
 
 		for (int n = 0; n < finfo.channels; ++n) {
 			try {
@@ -544,6 +669,9 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 				boost::shared_ptr<Source> s;
 
 				if ((s = session->source_by_path_and_channel (path, n)) == 0) {
+
+					cerr << "add embed/import source with defer_peaks = true\n";
+
 					source = boost::dynamic_pointer_cast<AudioFileSource> (SourceFactory::createReadable 
 											       (*session, path,  n,
 												(mode == ImportAsTapeTrack ? 
@@ -573,7 +701,7 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 	ret = add_sources (paths, sources, pos, mode, target_regions, target_tracks, track, true);
 
   out:
-	track_canvas.get_window()->set_cursor (*current_canvas_cursor);
+	track_canvas->get_window()->set_cursor (*current_canvas_cursor);
 	return ret;
 }
 
@@ -751,6 +879,13 @@ Editor::import_thread ()
 gint
 Editor::import_progress_timeout (void *arg)
 {
+	bool reset = false;
+
+	if (!interthread_progress_window->is_visible()) {
+		interthread_progress_window->show_all ();
+		reset = true;
+	}
+
 	interthread_progress_label.set_text (import_status.doing_what);
 
 	if (import_status.freeze) {
@@ -763,9 +898,20 @@ Editor::import_progress_timeout (void *arg)
 		interthread_progress_bar.pulse ();
 		return FALSE;
 	} else {
-		interthread_progress_bar.set_fraction (import_status.progress);
+		float val = import_status.progress;
+		interthread_progress_bar.set_fraction (min (max (0.0f, val), 1.0f));
 	}
 
-	return !(import_status.done || import_status.cancel);
+	if (reset) {
+
+		/* the window is now visible, speed up the updates */
+		
+		interthread_progress_connection.disconnect ();
+		interthread_progress_connection = Glib::signal_timeout().connect 
+			(bind (mem_fun(*this, &Editor::import_progress_timeout), (gpointer) 0), 100);
+		return false;
+	} else {
+		return !(import_status.done || import_status.cancel);
+	}
 }
 
