@@ -87,28 +87,37 @@ static void ardour_canvas_type_init()
 void
 Editor::initialize_canvas ()
 {
+	if (getenv ("ARDOUR_NON_AA_CANVAS")) {
+		track_canvas = new ArdourCanvas::Canvas ();
+		time_canvas = new ArdourCanvas::Canvas ();
+	} else {
+		track_canvas = new ArdourCanvas::CanvasAA ();
+		time_canvas = new ArdourCanvas::CanvasAA ();
+	}
+	
 	ArdourCanvas::init ();
 	ardour_canvas_type_init ();
 
 	/* don't try to center the canvas */
 
-	track_canvas.set_center_scroll_region (false);
-	track_canvas.set_dither  	(Gdk::RGB_DITHER_NONE);
+	track_canvas->set_center_scroll_region (false);
+	track_canvas->set_dither  	(Gdk::RGB_DITHER_NONE);
 
 	/* need to handle 4 specific types of events as catch-alls */
 
-	track_canvas.signal_scroll_event().connect (mem_fun (*this, &Editor::track_canvas_scroll_event));
-	track_canvas.signal_motion_notify_event().connect (mem_fun (*this, &Editor::track_canvas_motion_notify_event));
-	track_canvas.signal_button_press_event().connect (mem_fun (*this, &Editor::track_canvas_button_press_event));
-	track_canvas.signal_button_release_event().connect (mem_fun (*this, &Editor::track_canvas_button_release_event));
+	track_canvas->signal_scroll_event().connect (mem_fun (*this, &Editor::track_canvas_scroll_event));
+	track_canvas->signal_motion_notify_event().connect (mem_fun (*this, &Editor::track_canvas_motion_notify_event));
+	track_canvas->signal_button_press_event().connect (mem_fun (*this, &Editor::track_canvas_button_press_event));
+	track_canvas->signal_button_release_event().connect (mem_fun (*this, &Editor::track_canvas_button_release_event));
 
 	/* just scroll stuff for the timecanvas */
-	time_canvas.signal_scroll_event().connect (mem_fun (*this, &Editor::time_canvas_scroll_event));
+	time_canvas->signal_scroll_event().connect (mem_fun (*this, &Editor::time_canvas_scroll_event));
 
-	track_canvas.set_name ("EditorMainCanvas");
-	track_canvas.add_events (Gdk::POINTER_MOTION_HINT_MASK|Gdk::SCROLL_MASK);
-	track_canvas.signal_leave_notify_event().connect (mem_fun(*this, &Editor::left_track_canvas));
-	track_canvas.set_flags (CAN_FOCUS);
+	track_canvas->set_name ("EditorMainCanvas");
+	track_canvas->add_events (Gdk::POINTER_MOTION_HINT_MASK|Gdk::SCROLL_MASK);
+	track_canvas->signal_leave_notify_event().connect (mem_fun(*this, &Editor::left_track_canvas));
+	track_canvas->signal_enter_notify_event().connect (mem_fun(*this, &Editor::entered_track_canvas));
+	track_canvas->set_flags (CAN_FOCUS);
 
 	/* set up drag-n-drop */
 
@@ -121,14 +130,14 @@ Editor::initialize_canvas ()
 	target_table.push_back (TargetEntry ("text/uri-list"));
 	target_table.push_back (TargetEntry ("application/x-rootwin-drop"));
 
-	track_canvas.drag_dest_set (target_table);
-	track_canvas.signal_drag_data_received().connect (mem_fun(*this, &Editor::track_canvas_drag_data_received));
+	track_canvas->drag_dest_set (target_table);
+	track_canvas->signal_drag_data_received().connect (mem_fun(*this, &Editor::track_canvas_drag_data_received));
 
 	/* stuff for the verbose canvas cursor */
 
 	Pango::FontDescription* font = get_font_for_style (N_("VerboseCanvasCursor"));
 
-	verbose_canvas_cursor = new ArdourCanvas::Text (*track_canvas.root());
+	verbose_canvas_cursor = new ArdourCanvas::Text (*track_canvas->root());
 	verbose_canvas_cursor->property_font_desc() = *font;
 	verbose_canvas_cursor->property_anchor() = ANCHOR_NW;
 
@@ -140,7 +149,7 @@ Editor::initialize_canvas ()
 	
 	if (Profile->get_sae()) {
 		Image img (::get_icon (X_("saelogo")));
-		logo_item = new ArdourCanvas::Pixbuf (*track_canvas.root(), 0.0, 0.0, img.get_pixbuf());
+		logo_item = new ArdourCanvas::Pixbuf (*track_canvas->root(), 0.0, 0.0, img.get_pixbuf());
 		// logo_item->property_height_in_pixels() = true;
 		// logo_item->property_width_in_pixels() = true;
 		// logo_item->property_height_set() = true;
@@ -150,43 +159,52 @@ Editor::initialize_canvas ()
 	
 	/* a group to hold time (measure) lines */
 	
-	time_line_group = new ArdourCanvas::Group (*track_canvas.root(), 0.0, 0.0);
-	cursor_group = new ArdourCanvas::Group (*track_canvas.root(), 0.0, 0.0);
+	time_line_group = new ArdourCanvas::Group (*track_canvas->root(), 0.0, 0.0);
+	cursor_group = new ArdourCanvas::Group (*track_canvas->root(), 0.0, 0.0);
 
-	time_canvas.set_name ("EditorTimeCanvas");
-	time_canvas.add_events (Gdk::POINTER_MOTION_HINT_MASK);
-	time_canvas.set_flags (CAN_FOCUS);
-	time_canvas.set_center_scroll_region (false);
-	time_canvas.set_dither  	(Gdk::RGB_DITHER_NONE);
+	time_canvas->set_name ("EditorTimeCanvas");
+	time_canvas->add_events (Gdk::POINTER_MOTION_HINT_MASK);
+	time_canvas->set_flags (CAN_FOCUS);
+	time_canvas->set_center_scroll_region (false);
+	time_canvas->set_dither  	(Gdk::RGB_DITHER_NONE);
 	
-	meter_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, 0.0);
-	tempo_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height);
-	range_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 2.0);
-	transport_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 3.0);
-	marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 4.0);
-	cd_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 5.0);
+	meter_group = new ArdourCanvas::Group (*time_canvas->root(), 0.0, 0.0);
+	tempo_group = new ArdourCanvas::Group (*time_canvas->root(), 0.0, timebar_height);
+	range_marker_group = new ArdourCanvas::Group (*time_canvas->root(), 0.0, timebar_height * 2.0);
+	transport_marker_group = new ArdourCanvas::Group (*time_canvas->root(), 0.0, timebar_height * 3.0);
+	marker_group = new ArdourCanvas::Group (*time_canvas->root(), 0.0, timebar_height * 4.0);
+	cd_marker_group = new ArdourCanvas::Group (*time_canvas->root(), 0.0, timebar_height * 5.0);
 	
 	tempo_bar = new ArdourCanvas::SimpleRect (*tempo_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	tempo_bar->property_outline_what() = (0x1 | 0x8);
 	tempo_bar->property_outline_pixels() = 0;
 	
 	meter_bar = new ArdourCanvas::SimpleRect (*meter_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	meter_bar->property_outline_what() = (0x1 | 0x8);
 	meter_bar->property_outline_pixels() = 0;
 	
 	marker_bar = new ArdourCanvas::SimpleRect (*marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	marker_bar->property_outline_what() = (0x1 | 0x8);
 	marker_bar->property_outline_pixels() = 0;
 
 	cd_marker_bar = new ArdourCanvas::SimpleRect (*cd_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	cd_marker_bar->property_outline_pixels() = 0;
+ 	cd_marker_bar->property_outline_what() = (0x1 | 0x8);
+ 	cd_marker_bar->property_outline_pixels() = 0;
 	
 	range_marker_bar = new ArdourCanvas::SimpleRect (*range_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	range_marker_bar->property_outline_what() = (0x1 | 0x8);
 	range_marker_bar->property_outline_pixels() = 0;
 	
 	transport_marker_bar = new ArdourCanvas::SimpleRect (*transport_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	transport_marker_bar->property_outline_what() = (0x1 | 0x8);
 	transport_marker_bar->property_outline_pixels() = 0;
 	
+	cd_marker_bar_drag_rect = new ArdourCanvas::SimpleRect (*cd_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	cd_marker_bar_drag_rect->property_outline_pixels() = 0;
+	cd_marker_bar_drag_rect->hide ();
+
 	range_bar_drag_rect = new ArdourCanvas::SimpleRect (*range_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	range_bar_drag_rect->property_outline_pixels() = 0;
-	range_bar_drag_rect->hide ();
 	
 	transport_bar_drag_rect = new ArdourCanvas::SimpleRect (*transport_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	transport_bar_drag_rect->property_outline_pixels() = 0;
@@ -195,12 +213,12 @@ Editor::initialize_canvas ()
 	marker_drag_line_points.push_back(Gnome::Art::Point(0.0, 0.0));
 	marker_drag_line_points.push_back(Gnome::Art::Point(0.0, 0.0));
 
-	marker_drag_line = new ArdourCanvas::Line (*track_canvas.root());
+	marker_drag_line = new ArdourCanvas::Line (*track_canvas->root());
 	marker_drag_line->property_width_pixels() = 1;
 	marker_drag_line->property_points() = marker_drag_line_points;
 	marker_drag_line->hide();
 
-	range_marker_drag_rect = new ArdourCanvas::SimpleRect (*track_canvas.root(), 0.0, 0.0, 0.0, 0.0);
+	range_marker_drag_rect = new ArdourCanvas::SimpleRect (*track_canvas->root(), 0.0, 0.0, 0.0, 0.0);
 	range_marker_drag_rect->hide ();
 	
 	transport_loop_range_rect = new ArdourCanvas::SimpleRect (*time_line_group, 0.0, 0.0, 0.0, 0.0);
@@ -228,14 +246,14 @@ Editor::initialize_canvas ()
 	transport_punchout_line->hide();
 	
 	// used to show zoom mode active zooming
-	zoom_rect = new ArdourCanvas::SimpleRect (*track_canvas.root(), 0.0, 0.0, 0.0, 0.0);
+	zoom_rect = new ArdourCanvas::SimpleRect (*track_canvas->root(), 0.0, 0.0, 0.0, 0.0);
 	zoom_rect->property_outline_pixels() = 1;
 	zoom_rect->hide();
 	
 	zoom_rect->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_zoom_rect_event), (ArdourCanvas::Item*) 0));
 	
 	// used as rubberband rect
-	rubberband_rect = new ArdourCanvas::SimpleRect (*track_canvas.root(), 0.0, 0.0, 0.0, 0.0);
+	rubberband_rect = new ArdourCanvas::SimpleRect (*track_canvas->root(), 0.0, 0.0, 0.0, 0.0);
 	rubberband_rect->property_outline_pixels() = 1;
 	rubberband_rect->hide();
 	
@@ -251,12 +269,12 @@ Editor::initialize_canvas ()
 	
 	double time_height = timebar_height * 5;
 	double time_width = FLT_MAX/frames_per_unit;
-	time_canvas.set_scroll_region(0.0, 0.0, time_width, time_height);
+	time_canvas->set_scroll_region(0.0, 0.0, time_width, time_height);
 
 	playhead_cursor = new Cursor (*this, &Editor::canvas_playhead_cursor_event);
 
 	initial_ruler_update_required = true;
-	track_canvas.signal_size_allocate().connect (mem_fun(*this, &Editor::track_canvas_allocate));
+	track_canvas->signal_size_allocate().connect (mem_fun(*this, &Editor::track_canvas_allocate));
 
 	if (logo_item) {
 		logo_item->lower_to_bottom ();
@@ -325,15 +343,15 @@ Editor::track_canvas_size_allocated ()
 	}
 
  	range_marker_drag_rect->property_y1() = y1;
- 	range_marker_drag_rect->property_y2() = y1 + canvas_height;
+ 	range_marker_drag_rect->property_y2() = full_canvas_height;
  	transport_loop_range_rect->property_y1() = y1;
- 	transport_loop_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_loop_range_rect->property_y2() = full_canvas_height;
  	transport_punch_range_rect->property_y1() = y1;
- 	transport_punch_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_punch_range_rect->property_y2() = full_canvas_height;
  	transport_punchin_line->property_y1() = y1;
- 	transport_punchin_line->property_y2() = y1 + canvas_height;
+ 	transport_punchin_line->property_y2() = full_canvas_height;
  	transport_punchout_line->property_y1() = y1;
- 	transport_punchout_line->property_y2() = y1 + canvas_height;
+ 	transport_punchout_line->property_y2() = full_canvas_height;
 	
 	update_fixed_rulers();
 	redisplay_tempo (true);
@@ -359,10 +377,10 @@ Editor::reset_scrolling_region (Gtk::Allocation* alloc)
 
 	double last_canvas_unit =  max ((last_canvas_frame / frames_per_unit), canvas_width);
 
-	track_canvas.set_scroll_region (0.0, 0.0, last_canvas_unit, pos);
+	track_canvas->set_scroll_region (0.0, 0.0, last_canvas_unit, pos);
 
 	// XXX what is the correct height value for the time canvas ? this overstates it
-	time_canvas.set_scroll_region ( 0.0, 0.0, last_canvas_unit, canvas_height);
+	time_canvas->set_scroll_region ( 0.0, 0.0, last_canvas_unit, canvas_height);
 
 	range_marker_drag_rect->property_y2() = canvas_height;
 	transport_loop_range_rect->property_y2() = canvas_height;
@@ -418,14 +436,14 @@ Editor::controls_layout_size_request (Requisition* req)
 bool
 Editor::track_canvas_map_handler (GdkEventAny* ev)
 {
-	track_canvas.get_window()->set_cursor (*current_canvas_cursor);
+	track_canvas->get_window()->set_cursor (*current_canvas_cursor);
 	return false;
 }
 
 bool
 Editor::time_canvas_map_handler (GdkEventAny* ev)
 {
-	time_canvas.get_window()->set_cursor (*timebar_cursor);
+	time_canvas->get_window()->set_cursor (*timebar_cursor);
 	return false;
 }
 
@@ -435,6 +453,8 @@ Editor::track_canvas_drag_data_received (const RefPtr<Gdk::DragContext>& context
 					 const SelectionData& data,
 					 guint info, guint time)
 {
+	cerr << "drop on canvas, target = " << data.get_target() << endl;
+
 	if (data.get_target() == "regions") {
 		drop_regions (context, x, y, data, info, time);
 	} else {
@@ -466,7 +486,7 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 	double wx;
 	double wy;
 
-	track_canvas.window_to_world (x, y, wx, wy);
+	track_canvas->window_to_world (x, y, wx, wy);
 	wx += horizontal_adjustment.get_value();
 	wy += vertical_adjustment.get_value();
 	
@@ -484,7 +504,11 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 
 		frame = 0;
 
-		do_embed (paths, Editing::ImportDistinctFiles, ImportAsTrack, frame);
+		if (Profile->get_sae() || Config->get_only_copy_imported_files()) {
+			do_import (paths, Editing::ImportDistinctFiles, Editing::ImportAsTrack, SrcBest, frame); 
+		} else {
+			do_embed (paths, Editing::ImportDistinctFiles, ImportAsTrack, frame);
+		}
 		
 	} else if ((tv = dynamic_cast<AudioTimeAxisView*>(tvp)) != 0) {
 
@@ -493,7 +517,12 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 		if (tv->get_diskstream()) {
 			/* select the track, then embed */
 			selection->set (tv);
-			do_embed (paths, Editing::ImportDistinctFiles, ImportToTrack, frame);
+
+			if (Profile->get_sae() || Config->get_only_copy_imported_files()) {
+				do_import (paths, Editing::ImportDistinctFiles, Editing::ImportToTrack, SrcBest, frame); 
+			} else {
+				do_embed (paths, Editing::ImportDistinctFiles, ImportToTrack, frame);
+			}
 		}
 	}
 
@@ -524,47 +553,52 @@ Editor::drop_regions (const RefPtr<Gdk::DragContext>& context,
 }
 
 void
-Editor::maybe_autoscroll (GdkEvent* event)
+Editor::maybe_autoscroll (GdkEventMotion* event)
 {
 	nframes_t rightmost_frame = leftmost_frame + current_page_frames();
 	nframes_t frame = drag_info.current_pointer_frame;
 	bool startit = false;
+	double vertical_pos = vertical_adjustment.get_value();
 
-	static int last_autoscroll_direction = 0;
+	autoscroll_y = 0;
+	autoscroll_x = 0;
+
+	if (event->y < vertical_pos) {
+		autoscroll_y = -1;
+		startit = true;
+	}
+
+	if (event->y > vertical_pos + canvas_height) {
+		autoscroll_y = 1;
+		startit = true;
+	}
 
 	if (frame > rightmost_frame) {
 
 		if (rightmost_frame < max_frames) {
-			autoscroll_direction = 1;
+			autoscroll_x = 1;
 			startit = true;
 		}
 
 	} else if (frame < leftmost_frame) {
-
+		
 		if (leftmost_frame > 0) {
-			autoscroll_direction = -1;
+			autoscroll_x = -1;
 			startit = true;
 		}
 
-	} else {
-
-		if (drag_info.last_pointer_frame > drag_info.current_pointer_frame) {
-			autoscroll_direction = -1;
-		} else {
-			autoscroll_direction = 1;
-		}
 	}
 
-
-	if ((autoscroll_direction != last_autoscroll_direction) || (leftmost_frame < frame < rightmost_frame)) {
+	if ((autoscroll_x != last_autoscroll_x) || (autoscroll_y != last_autoscroll_y) || (autoscroll_x == 0 && autoscroll_y == 0)) {
 		stop_canvas_autoscroll ();
 	}
 	
 	if (startit && autoscroll_timeout_tag < 0) {
-		start_canvas_autoscroll (autoscroll_direction);
+		start_canvas_autoscroll (autoscroll_x, autoscroll_y);
 	}
 
-	last_autoscroll_direction = autoscroll_direction;
+	last_autoscroll_x = autoscroll_x;
+	last_autoscroll_y = autoscroll_y;
 }
 
 gint
@@ -580,21 +614,64 @@ Editor::autoscroll_canvas ()
 	nframes_t limit = max_frames - current_page_frames();
 	GdkEventMotion ev;
 	nframes_t target_frame;
+	double new_pixel;
+	double target_pixel;
 
-	if (autoscroll_direction < 0) {
-		if (leftmost_frame < autoscroll_distance) {
+	if (autoscroll_x < 0) {
+		if (leftmost_frame < autoscroll_x_distance) {
 			new_frame = 0;
 		} else {
-			new_frame = leftmost_frame - autoscroll_distance;
+			new_frame = leftmost_frame - autoscroll_x_distance;
 		}
-		target_frame = drag_info.current_pointer_frame - autoscroll_distance;
- 	} else {
-		if (leftmost_frame > limit - autoscroll_distance) {
+		target_frame = drag_info.current_pointer_frame - autoscroll_x_distance;
+ 	} else if (autoscroll_x > 0) {
+		if (leftmost_frame > limit - autoscroll_x_distance) {
 			new_frame = limit;
 		} else {
-			new_frame = leftmost_frame + autoscroll_distance;
+			new_frame = leftmost_frame + autoscroll_x_distance;
 		}
-		target_frame = drag_info.current_pointer_frame + autoscroll_distance;
+		target_frame = drag_info.current_pointer_frame + autoscroll_x_distance;
+	} else {
+		target_frame = drag_info.current_pointer_frame;
+		new_frame = leftmost_frame;
+	}
+
+	double vertical_pos = vertical_adjustment.get_value();
+
+	if (autoscroll_y < 0) {
+
+		if (vertical_pos < autoscroll_y_distance) {
+			new_pixel = 0;
+		} else {
+			new_pixel = vertical_pos - autoscroll_y_distance;
+		}
+
+		target_pixel = drag_info.current_pointer_y - autoscroll_y_distance;
+		target_pixel = max (target_pixel, 0.0);
+
+ 	} else if (autoscroll_y > 0) {
+
+		double top_of_bottom_of_canvas = full_canvas_height - canvas_height;
+
+		if (vertical_pos > full_canvas_height - autoscroll_y_distance) {
+			new_pixel = full_canvas_height;
+		} else {
+			new_pixel = vertical_pos + autoscroll_y_distance;
+		}
+
+		new_pixel = min (top_of_bottom_of_canvas, new_pixel);
+
+		target_pixel = drag_info.current_pointer_y + autoscroll_y_distance;
+		
+		/* don't move to the full canvas height because the item will be invisible
+		   (its top edge will line up with the bottom of the visible canvas.
+		*/
+
+		target_pixel = min (target_pixel, full_canvas_height - 10);
+		
+	} else {
+		target_pixel = drag_info.current_pointer_y;
+		new_pixel = vertical_pos;
 	}
 
 	/* now fake a motion event to get the object that is being dragged to move too */
@@ -602,10 +679,10 @@ Editor::autoscroll_canvas ()
 	ev.type = GDK_MOTION_NOTIFY;
 	ev.state &= Gdk::BUTTON1_MASK;
 	ev.x = frame_to_unit (target_frame);
-	ev.y = drag_info.current_pointer_y;
+	ev.y = target_pixel;
 	motion_handler (drag_info.item, (GdkEvent*) &ev, drag_info.item_type, true);
 
-	if (new_frame == 0 || new_frame == limit) {
+	if ((new_frame == 0 || new_frame == limit) && (new_pixel == 0 || new_pixel == DBL_MAX)) {
 		/* we are done */
 		return false;
 	}
@@ -625,29 +702,54 @@ Editor::autoscroll_canvas ()
 		reset_x_origin (new_frame);
 	}
 
-	if (autoscroll_cnt == 50) { /* 0.5 seconds */
-		
-		/* after about a while, speed up a bit by changing the timeout interval */
+	vertical_adjustment.set_value (new_pixel);
 
-		autoscroll_distance = (nframes_t) floor (current_page_frames()/30.0f);
-		
-	} else if (autoscroll_cnt == 150) { /* 1.0 seconds */
+	if (autoscroll_x_distance != 0) {
 
-		autoscroll_distance = (nframes_t) floor (current_page_frames()/20.0f);
+		if (autoscroll_cnt == 50) { /* 0.5 seconds */
+			
+			/* after about a while, speed up a bit by changing the timeout interval */
+			
+			autoscroll_x_distance = (nframes_t) floor (current_page_frames()/30.0f);
+			
+		} else if (autoscroll_cnt == 150) { /* 1.0 seconds */
+			
+			autoscroll_x_distance = (nframes_t) floor (current_page_frames()/20.0f);
+			
+		} else if (autoscroll_cnt == 300) { /* 1.5 seconds */
+			
+			/* after about another while, speed up by increasing the shift per callback */
+			
+			autoscroll_x_distance =  (nframes_t) floor (current_page_frames()/10.0f);
+			
+		} 
+	}
 
-	} else if (autoscroll_cnt == 300) { /* 1.5 seconds */
+	if (autoscroll_y_distance != 0) {
 
-		/* after about another while, speed up by increasing the shift per callback */
-
-		autoscroll_distance =  (nframes_t) floor (current_page_frames()/10.0f);
-
-	} 
+		if (autoscroll_cnt == 50) { /* 0.5 seconds */
+			
+			/* after about a while, speed up a bit by changing the timeout interval */
+			
+			autoscroll_y_distance = 10;
+			
+		} else if (autoscroll_cnt == 150) { /* 1.0 seconds */
+			
+			autoscroll_y_distance = 20;
+			
+		} else if (autoscroll_cnt == 300) { /* 1.5 seconds */
+			
+			/* after about another while, speed up by increasing the shift per callback */
+			
+			autoscroll_y_distance =  40;
+		} 
+	}
 
 	return true;
 }
 
 void
-Editor::start_canvas_autoscroll (int dir)
+Editor::start_canvas_autoscroll (int dx, int dy)
 {
 	if (!session || autoscroll_active) {
 		return;
@@ -656,8 +758,10 @@ Editor::start_canvas_autoscroll (int dir)
 	stop_canvas_autoscroll ();
 
 	autoscroll_active = true;
-	autoscroll_direction = dir;
-	autoscroll_distance = (nframes_t) floor (current_page_frames()/50.0);
+	autoscroll_x = dx;
+	autoscroll_y = dy;
+	autoscroll_x_distance = (nframes_t) floor (current_page_frames()/50.0);
+	autoscroll_y_distance = fabs (dy * 5); /* pixels */
 	autoscroll_cnt = 0;
 	
 	/* do it right now, which will start the repeated callbacks */
@@ -676,11 +780,19 @@ Editor::stop_canvas_autoscroll ()
 	autoscroll_active = false;
 }
 
-gint
+bool
 Editor::left_track_canvas (GdkEventCrossing *ev)
 {
 	set_entered_track (0);
 	set_entered_regionview (0);
+	reset_canvas_action_sensitivity (false);
+	return false;
+}
+
+bool
+Editor::entered_track_canvas (GdkEventCrossing *ev)
+{
+	reset_canvas_action_sensitivity (true);
 	return FALSE;
 }
 
@@ -692,15 +804,15 @@ Editor::tie_vertical_scrolling ()
 	playhead_cursor->set_y_axis (y1);
 
  	range_marker_drag_rect->property_y1() = y1;
- 	range_marker_drag_rect->property_y2() = y1 + canvas_height;
+ 	range_marker_drag_rect->property_y2() = full_canvas_height;
  	transport_loop_range_rect->property_y1() = y1;
- 	transport_loop_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_loop_range_rect->property_y2() = full_canvas_height;
  	transport_punch_range_rect->property_y1() = y1;
- 	transport_punch_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_punch_range_rect->property_y2() = full_canvas_height;
  	transport_punchin_line->property_y1() = y1;
- 	transport_punchin_line->property_y2() = y1 + canvas_height;
+ 	transport_punchin_line->property_y2() = full_canvas_height;
  	transport_punchout_line->property_y1() = y1;
- 	transport_punchout_line->property_y2() = y1 + canvas_height;
+ 	transport_punchout_line->property_y2() = full_canvas_height;
 
 	if (!selection->markers.empty()) {
 		for (MarkerSelection::iterator x = selection->markers.begin(); x != selection->markers.end(); ++x) {		
@@ -771,6 +883,9 @@ Editor::color_handler()
 	transport_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportMarkerBar.get();
 	transport_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
 
+	cd_marker_bar_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
+	cd_marker_bar_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
+
 	range_bar_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
 	range_bar_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
 
@@ -814,8 +929,8 @@ void
 Editor::flush_canvas ()
 {
 	if (is_mapped()) {
-		track_canvas.update_now ();
-		// gdk_window_process_updates (GTK_LAYOUT(track_canvas.gobj())->bin_window, true);
+		track_canvas->update_now ();
+		// gdk_window_process_updates (GTK_LAYOUT(track_canvas->gobj())->bin_window, true);
 	}
 }
 
