@@ -25,6 +25,7 @@
 #include <ardour/diskstream.h>
 #include <ardour/playlist.h>
 #include <ardour/route_group.h>
+#include <ardour/profile.h>
 
 #include "editor.h"
 #include "actions.h"
@@ -170,6 +171,7 @@ Editor::set_selected_track_as_side_effect (bool force)
 	}
 
 	if (!selection->tracks.empty()) {
+
 		if (!selection->selected (clicked_trackview)) {
 			selection->add (clicked_trackview);
 		}
@@ -182,7 +184,6 @@ Editor::set_selected_track_as_side_effect (bool force)
 void
 Editor::set_selected_track (TimeAxisView& view, Selection::Operation op, bool no_remove)
 {
-
 	switch (op) {
 	case Selection::Toggle:
 		if (selection->selected (&view)) {
@@ -201,21 +202,7 @@ Editor::set_selected_track (TimeAxisView& view, Selection::Operation op, bool no
 		break;
 
 	case Selection::Set:
-		if (selection->selected (&view) && selection->tracks.size() > 1) {
-
-			/* reset track selection if there is only 1 other track
-			   selected OR if no_remove is not set (its there to 
-			   prevent deselecting a multi-track selection
-			   when clicking on an already selected track
-			   for some reason.
-			*/
-
-			if (selection->tracks.empty()) {
-				selection->set (&view);
-			} else if (selection->tracks.size() == 1 || !no_remove) {
-				selection->set (&view);
-			}
-		}
+		selection->set (&view);
 		break;
 		
 	case Selection::Extend:
@@ -764,11 +751,17 @@ Editor::track_selection_changed ()
 			(*i)->set_selected (false);
 		}
 	}
+
+	ActionManager::set_sensitive (ActionManager::track_selection_sensitive_actions, !selection->tracks.empty());
 }
 
 void
 Editor::time_selection_changed ()
 {
+	if (Profile->get_sae()) {
+		return;
+	}
+
 	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
 		(*i)->hide_selection ();
 	}
@@ -791,6 +784,30 @@ Editor::time_selection_changed ()
 
 }
 
+void
+Editor::sensitize_the_right_region_actions (bool have_selected_regions)
+{
+	for (vector<Glib::RefPtr<Action> >::iterator x = ActionManager::region_selection_sensitive_actions.begin();
+	     x != ActionManager::region_selection_sensitive_actions.end(); ++x) {
+
+		string accel_path = (*x)->get_accel_path ();
+		AccelKey key;
+
+		/* if there is an accelerator, it should always be sensitive
+		   to allow for keyboard ops on entered regions.
+		*/
+
+		bool known = ActionManager::lookup_entry (accel_path, key);
+
+		if (known && ((key.get_key() != GDK_VoidSymbol) && (key.get_key() != 0))) {
+			(*x)->set_sensitive (true);
+		} else {
+			(*x)->set_sensitive (have_selected_regions);
+		}
+	}
+}
+
+
 void
 Editor::region_selection_changed ()
 {
@@ -798,6 +815,8 @@ Editor::region_selection_changed ()
 		(*i)->set_selected_regionviews (selection->regions);
 	}
 	
+	sensitize_the_right_region_actions (!selection->regions.empty());
+
 	zoomed_to_region = false;
 }
 
@@ -964,14 +983,10 @@ Editor::set_selection_from_audio_region ()
 		return;
 	}
 
-	RegionView* rv = *(selection->regions.begin());
-	boost::shared_ptr<Region> region = rv->region();
-	
-	begin_reversible_command (_("set selection from region"));
-	selection->set (0, region->position(), region->last_frame());
-	commit_reversible_command ();
-
-	set_mouse_mode (Editing::MouseRange, false);
+	selection->set (0, selection->regions.start(), selection->regions.end_frame());
+	if (!Profile->get_sae()) {
+		set_mouse_mode (Editing::MouseRange, false);
+	}
 }
 
 void
@@ -1004,7 +1019,9 @@ Editor::set_selection_from_range (Location& loc)
 	selection->set (0, loc.start(), loc.end());
 	commit_reversible_command ();
 
-	set_mouse_mode (Editing::MouseRange, false);
+	if (!Profile->get_sae()) {
+		set_mouse_mode (Editing::MouseRange, false);
+	}
 }
 
 void
@@ -1317,6 +1334,12 @@ Editor::get_edit_op_range (nframes64_t& start, nframes64_t& end) const
 		swap (start, end);
 	}
 
+	/* turn range into one delimited by start...end,
+	   not start...end-1
+	*/
+
+	end++;
+
 	return true;
 }
 
@@ -1326,27 +1349,4 @@ Editor::deselect_all ()
 	selection->clear ();
 }
 
-Editor::ExclusiveRegionSelection::ExclusiveRegionSelection (Editor& ed, RegionView* rv)
-	: editor (ed),
-	  regionview (rv)
-{
-
-	if (!rv || ed.current_mouse_mode() != Editing::MouseObject) {
-		return;
-	}
-	
-	if (ed.get_selection().regions.empty() && !ed.get_selection().selected (rv)) {
-		ed.get_selection().set (rv, false);
-		remove = true;
-	} else {
-		remove = false;
-	}
-}
-
-Editor::ExclusiveRegionSelection::~ExclusiveRegionSelection ()
-{
-	if (remove) {
-		editor.get_selection().remove (regionview);
-	}
-}
 
