@@ -90,6 +90,8 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 						     _io->gain_control(),
 						     false));
 
+	level_meter = new LevelMeter(_io, _session);
+
 	gain_slider->signal_button_press_event().connect (mem_fun(*this, &GainMeter::start_gain_touch));
 	gain_slider->signal_button_release_event().connect (mem_fun(*this, &GainMeter::end_gain_touch));
 	gain_slider->set_name ("GainFader");
@@ -113,11 +115,9 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	peak_display.set_label (_("-inf"));
 	peak_display.unset_flags (Gtk::CAN_FOCUS);
 
-	meter_metric_area.set_name ("MeterMetricsStrip");
+	meter_metric_area.set_name ("AudioTrackMetrics");
 	set_size_request_to_display_given_text (meter_metric_area, "-50", 0, 0);
 
-	meter_packer.set_spacing (2);
-
 	gain_automation_style_button.set_name ("MixerAutomationModeButton");
 	gain_automation_state_button.set_name ("MixerAutomationPlaybackButton");
 
@@ -140,46 +140,46 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	hbox.set_spacing (2);
 	hbox.pack_start (*fader_vbox, true, true);
 
-	set_width(Narrow);
-
 	Route* r;
 
 	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
 
-	        /* 
-		   if we have a route (ie. we're not the click), 
+		/* 
+		   if we have a non-hidden route (ie. we're not the click or the auditioner), 
 		   pack some route-dependent stuff.
 		*/
 
-	        gain_display_box.pack_end (peak_display, true, true);
+		gain_display_box.pack_end (peak_display, true, true);
+		hbox.pack_end (*level_meter, true, true);
 
-		hbox.pack_end (meter_packer, true, true);
+		if (!r->hidden()) {
 
-		using namespace Menu_Helpers;
-	
-		gain_astate_menu.items().push_back (MenuElem (_("Manual"), 
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Off)));
-		gain_astate_menu.items().push_back (MenuElem (_("Play"),
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Play)));
-		gain_astate_menu.items().push_back (MenuElem (_("Write"),
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Write)));
-		gain_astate_menu.items().push_back (MenuElem (_("Touch"),
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Touch)));
-	
-		gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
-		gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
-	
-		gain_astate_menu.set_name ("ArdourContextMenu");
-		gain_astyle_menu.set_name ("ArdourContextMenu");
-
-		gain_automation_style_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_style_button_event), false);
-		gain_automation_state_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_state_button_event), false);
-		
-		r->gain_automation_curve().automation_state_changed.connect (mem_fun(*this, &GainMeter::gain_automation_state_changed));
-		r->gain_automation_curve().automation_style_changed.connect (mem_fun(*this, &GainMeter::gain_automation_style_changed));
-		fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
-
-		gain_automation_state_changed ();
+			using namespace Menu_Helpers;
+			
+			gain_astate_menu.items().push_back (MenuElem (_("Manual"), 
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Off)));
+			gain_astate_menu.items().push_back (MenuElem (_("Play"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Play)));
+			gain_astate_menu.items().push_back (MenuElem (_("Write"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Write)));
+			gain_astate_menu.items().push_back (MenuElem (_("Touch"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Touch)));
+			
+			gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
+			gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
+			
+			gain_astate_menu.set_name ("ArdourContextMenu");
+			gain_astyle_menu.set_name ("ArdourContextMenu");
+			
+			gain_automation_style_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_style_button_event), false);
+			gain_automation_state_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_state_button_event), false);
+			
+			r->gain_automation_curve().automation_state_changed.connect (mem_fun(*this, &GainMeter::gain_automation_state_changed));
+			r->gain_automation_curve().automation_style_changed.connect (mem_fun(*this, &GainMeter::gain_automation_style_changed));
+			
+			fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
+			gain_automation_state_changed ();
+		}
 	}
 
 	set_spacing (2);
@@ -194,8 +194,6 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	peak_display.signal_button_release_event().connect (mem_fun(*this, &GainMeter::peak_button_release), false);
 	gain_display.signal_key_press_event().connect (mem_fun(*this, &GainMeter::gain_key_press), false);
 
-	Config->ParameterChanged.connect (mem_fun (*this, &GainMeter::parameter_changed));
-
 	gain_changed (0);
 	show_gain ();
 
@@ -205,27 +203,24 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	ResetGroupPeakDisplays.connect (mem_fun(*this, &GainMeter::reset_group_peak_display));
 
 	UI::instance()->theme_changed.connect (mem_fun(*this, &GainMeter::on_theme_changed));
+	ColorsChanged.connect (bind(mem_fun (*this, &GainMeter::color_handler), false));
+	DPIReset.connect (bind(mem_fun (*this, &GainMeter::color_handler), true));
+	//hide_all();
 }
 
 void
-GainMeter::set_width (Width w)
+GainMeter::set_width (Width w, int len)
 {
-	switch (w) {
-	case Wide:
-		peak_display.show();
-		break;
-	case Narrow:
-		peak_display.hide();
-		break;
-	}
-
 	_width = w;
-	setup_meters ();
+	level_meter->setup_meters (len);
 }
 
 Glib::RefPtr<Gdk::Pixmap>
 GainMeter::render_metrics (Gtk::Widget& w)
 {
+	//cerr << "name: " << w.get_name();//DEBUG
+	//cerr << " bg_red = " << w.get_style()->get_bg(Gtk::STATE_NORMAL).get_red();//DEBUG
+	//cerr << " fg_red = " << w.get_style()->get_fg(Gtk::STATE_NORMAL).get_red();//DEBUG
 	Glib::RefPtr<Gdk::Window> win (w.get_window());
 	Glib::RefPtr<Gdk::GC> fg_gc (w.get_style()->get_fg_gc (Gtk::STATE_NORMAL));
 	Glib::RefPtr<Gdk::GC> bg_gc (w.get_style()->get_bg_gc (Gtk::STATE_NORMAL));
@@ -268,13 +263,15 @@ gint
 GainMeter::meter_metrics_expose (GdkEventExpose *ev)
 {
 	static Glib::RefPtr<Gtk::Style> meter_style;
-
+	//cerr << "GainMeter::meter_metrics_expose() called on " << meter_metric_area.get_name();//DEBUG
+	//cerr << " style_changed = " << style_changed;//DEBUG
 	if (style_changed) {
 		meter_style = meter_metric_area.get_style();
+	//	cerr << " get_style() called ";//DEBUG
 	}
 	Glib::RefPtr<Gdk::Window> win (meter_metric_area.get_window());
-	Glib::RefPtr<Gdk::GC> fg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
-	Glib::RefPtr<Gdk::GC> bg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
+	//Glib::RefPtr<Gdk::GC> fg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
+	Glib::RefPtr<Gdk::GC> bg_gc (meter_style->get_bg_gc (Gtk::STATE_INSENSITIVE));
 	GdkRectangle base_rect;
 	GdkRectangle draw_rect;
 	gint width, height;
@@ -289,16 +286,17 @@ GainMeter::meter_metrics_expose (GdkEventExpose *ev)
 	Glib::RefPtr<Gdk::Pixmap> pixmap;
 	std::map<string,Glib::RefPtr<Gdk::Pixmap> >::iterator i = metric_pixmaps.find (meter_metric_area.get_name());
 
-	if (i == metric_pixmaps.end() || style_changed) {
+	if (i == metric_pixmaps.end() || style_changed || dpi_changed) {
 		pixmap = render_metrics (meter_metric_area);
 	} else {
 		pixmap = i->second;
+	//	cerr << " re-used " << i->first;//DEBUG
 	}
 
 	gdk_rectangle_intersect (&ev->area, &base_rect, &draw_rect);
-	win->draw_rectangle (bg_gc, true, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
-	win->draw_drawable (fg_gc, pixmap, draw_rect.x, draw_rect.y, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
-	
+	//win->draw_rectangle (bg_gc, false, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
+	win->draw_drawable (bg_gc, pixmap, draw_rect.x, draw_rect.y, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
+	//cerr << endl;//DEBUG
 	style_changed = false;
 	return true;
 }
@@ -315,152 +313,34 @@ GainMeter::~GainMeter ()
 		delete meter_menu;
 	}
 
-	for (vector<MeterInfo>::iterator i = meters.begin(); i != meters.end(); i++) {
-		if ((*i).meter) {
-			delete (*i).meter;
-		}
+	if (level_meter) {
+		delete level_meter;
 	}
 }
 
-void
-GainMeter::update_meters ()
-{
-	vector<MeterInfo>::iterator i;
-	uint32_t n;
-	float peak, mpeak;
-	char buf[32];
-	
-	for (n = 0, i = meters.begin(); i != meters.end(); ++i, ++n) {
-		if ((*i).packed) {
-			peak = _io->peak_input_power (n);
-
-			(*i).meter->set (log_meter (peak));
-
-			mpeak = _io->max_peak_power(n);
-			
-			if (mpeak > max_peak) {
-				max_peak = mpeak;
-				/* set peak display */
-				if (max_peak <= -200.0f) {
-					peak_display.set_label (_("-inf"));
-				} else {
-					snprintf (buf, sizeof(buf), "%.1f", max_peak);
-					peak_display.set_label (buf);
-				}
-
-				if (max_peak >= 0.0f) {
-					peak_display.set_name ("MixerStripPeakDisplayPeak");
-				}
-			}
-		}
-	}
-}
-
-void
-GainMeter::parameter_changed(const char* parameter_name)
-{
-#define PARAM_IS(x) (!strcmp (parameter_name, (x)))
-
-	ENSURE_GUI_THREAD (bind (mem_fun(*this, &GainMeter::parameter_changed), parameter_name));
-
-	if (PARAM_IS ("meter-hold")) {
-	
-		vector<MeterInfo>::iterator i;
-		uint32_t n;
-		
-		for (n = 0, i = meters.begin(); i != meters.end(); ++i, ++n) {
-			
-			(*i).meter->set_hold_count ((uint32_t) floor(Config->get_meter_hold()));
-		}
-	}
-
-#undef PARAM_IS
-}
-
 void
 GainMeter::hide_all_meters ()
 {
 	bool remove_metric_area = false;
 
-	for (vector<MeterInfo>::iterator i = meters.begin(); i != meters.end(); ++i) {
-		if ((*i).packed) {
-			remove_metric_area = true;
-			meter_packer.remove (*((*i).meter));
-			(*i).packed = false;
-		}
-	}
+	level_meter->hide_meters();
 
 	if (remove_metric_area) {
 		if (meter_metric_area.get_parent()) {
-			meter_packer.remove (meter_metric_area);
+			level_meter->remove (meter_metric_area);
 		}
 	}
 }
 
 void
-GainMeter::setup_meters ()
+GainMeter::setup_meters (int len)
 {
-	uint32_t nmeters = _io->n_outputs();
-	guint16 width;
-
-	hide_all_meters ();
-
-	Route* r;
-
-	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
-
-		switch (r->meter_point()) {
-		case MeterPreFader:
-		case MeterInput:
-			nmeters = r->n_inputs();
-			break;
-		case MeterPostFader:
-			nmeters = r->n_outputs();
-			break;
-		}
-
-	} else {
-
-		nmeters = _io->n_outputs();
-
-	}
-
-	if (nmeters == 0) {
-		return;
-	}
-
-	if (nmeters <= 2) {
-		width = regular_meter_width;
-	} else {
-		width = thin_meter_width;
-	}
-
-	while (meters.size() < nmeters) {
-		meters.push_back (MeterInfo());
-	}
-
-	/* pack them backwards */
-
-	if (_width == Wide) {
-	        meter_packer.pack_end (meter_metric_area, false, false);
+	if (!meter_metric_area.get_parent()) {
+		level_meter->pack_end (meter_metric_area, false, false);
 		meter_metric_area.show_all ();
 	}
-
-	for (int32_t n = nmeters-1; nmeters && n >= 0 ; --n) {
-		if (meters[n].width != width) {
-			delete meters[n].meter;
-			meters[n].meter = new FastMeter ((uint32_t) floor (Config->get_meter_hold()), width, FastMeter::Vertical);
-			meters[n].width = width;
-
-			meters[n].meter->add_events (Gdk::BUTTON_RELEASE_MASK);
-			meters[n].meter->signal_button_release_event().connect (bind (mem_fun(*this, &GainMeter::meter_button_release), n));
-		}
-
-		meter_packer.pack_end (*meters[n].meter, false, false);
-		meters[n].meter->show_all ();
-		meters[n].packed = true;
-	}
-}	
+	level_meter->setup_meters(len, 5);
+}
 
 int
 GainMeter::get_gm_width ()
@@ -486,9 +366,9 @@ GainMeter::peak_button_release (GdkEventButton* ev)
 {
 	/* reset peak label */
 
-	if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::Control|Keyboard::Shift)) {
+	if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier|Keyboard::TertiaryModifier)) {
 		ResetAllPeakDisplays ();
-	} else if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+	} else if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 		Route* r;
 		if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
 			ResetGroupPeakDisplays (r->mix_group());
@@ -508,6 +388,7 @@ GainMeter::reset_peak_display ()
 		r->reset_max_peak_meters();
 	}
 
+	level_meter->clear_meters();
 	max_peak = -INFINITY;
 	peak_display.set_label (_("-Inf"));
 	peak_display.set_name ("MixerStripPeakDisplay");
@@ -524,25 +405,6 @@ GainMeter::reset_group_peak_display (RouteGroup* group)
 	}
 }
 
-gint
-GainMeter::meter_button_release (GdkEventButton* ev, uint32_t which)
-{
-	switch (ev->button) {
-	case 1:
-		meters[which].meter->clear();
-		max_peak = minus_infinity();
-		peak_display.set_label (_("-inf"));
-		peak_display.set_name ("MixerStripPeakDisplay");
-		break;
-
-	case 3:
-		// popup_meter_menu (ev);
-		break;
-	};
-
-	return TRUE;
-}
-
 void
 GainMeter::popup_meter_menu (GdkEventButton *ev)
 {
@@ -623,7 +485,7 @@ void
 GainMeter::effective_gain_display ()
 {
 	gfloat value = gain_to_slider_position (_io->effective_gain());
-	
+
 	if (gain_adjustment.get_value() != value) {
 		ignore_toggle = true; 
 		gain_adjustment.set_value (value);
@@ -641,6 +503,7 @@ void
 GainMeter::set_meter_strip_name (const char * name)
 {
 	meter_metric_area.set_name (name);
+	style_changed = true;
 }
 
 void
@@ -697,10 +560,10 @@ GainMeter::meter_press(GdkEventButton* ev)
 
 			if (ev->button == 2) {
 
-				// ctrl-button2 click is the midi binding click
+				// Primary-button2 click is the midi binding click
 				// button2-click is "momentary"
 				
-				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::Control))) {
+				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
 					wait_for_release = true;
 					old_meter_point = _route->meter_point ();
 				}
@@ -708,9 +571,9 @@ GainMeter::meter_press(GdkEventButton* ev)
 
 			if (ev->button == 1 || ev->button == 2) {
 
-				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::Control|Keyboard::Shift))) {
+				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
 
-					/* ctrl-shift-click applies change to all routes */
+					/* Primary+Tertiary-click applies change to all routes */
 
 					_session.begin_reversible_command (_("meter point change"));
                                         Session::GlobalMeteringStateCommand *cmd = new Session::GlobalMeteringStateCommand (_session, this);
@@ -720,10 +583,10 @@ GainMeter::meter_press(GdkEventButton* ev)
 					_session.commit_reversible_command ();
 					
 					
-				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 
-					/* ctrl-click: solo mix group.
-					   ctrl-button2 is MIDI learn.
+					/* Primary-click: solo mix group.
+					   NOTE: Primary-button2 is MIDI learn.
 					*/
 					
 					if (ev->button == 1) {
@@ -949,3 +812,31 @@ GainMeter::gain_automation_state_changed ()
 		gain_watching = ARDOUR_UI::RapidScreenUpdate.connect (mem_fun (*this, &GainMeter::effective_gain_display));
 	}
 }
+
+void
+GainMeter::update_meters()
+{
+	char buf[32];
+	float mpeak = level_meter->update_meters();
+
+	if (mpeak > max_peak) {
+		max_peak = mpeak;
+		if (mpeak <= -200.0f) {
+			peak_display.set_label (_("-inf"));
+		} else {
+			snprintf (buf, sizeof(buf), "%.1f", mpeak);
+			peak_display.set_label (buf);
+		}
+
+		if (mpeak >= 0.0f) {
+			peak_display.set_name ("MixerStripPeakDisplayPeak");
+		}
+	}
+}
+
+void GainMeter::color_handler(bool dpi)
+{
+	color_changed = true;
+	dpi_changed = (dpi) ? true : false;
+	setup_meters();
+}
