@@ -42,6 +42,7 @@
 #include <ardour/panner.h>
 #include <ardour/send.h>
 #include <ardour/insert.h>
+#include <ardour/profile.h>
 #include <ardour/ladspa_plugin.h>
 #include <ardour/connection.h>
 #include <ardour/session_connection.h>
@@ -86,6 +87,7 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	: AxisView(sess),
 	  RouteUI (rt, sess, _("Mute"), _("Solo"), _("Record")),
 	  _mixer(mx),
+	  _mixer_owned (in_mixer),
 	  pre_redirect_box (PreFader, sess, rt, mx.plugin_selector(), mx.selection(), in_mixer),
 	  post_redirect_box (PostFader, sess, rt, mx.plugin_selector(), mx.selection(), in_mixer),
 	  gpm (_route, sess),
@@ -185,7 +187,7 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 
 		rec_enable_button->set_name ("MixerRecordEnableButton");
 
-		AudioTrack* at = audio_track();
+		boost::shared_ptr<AudioTrack> at = audio_track();
 
 		at->FreezeChange.connect (mem_fun(*this, &MixerStrip::map_frozen));
 
@@ -402,10 +404,6 @@ MixerStrip::set_width (Width w, void* owner)
 
 	_width_owner = owner;
 
-	if (_width == w) {
-		return;
-	}
-
 	ensure_xml_node ();
 	
 	_width = w;
@@ -664,12 +662,11 @@ MixerStrip::add_connection_to_output_menu (ARDOUR::Connection* c)
 void
 MixerStrip::update_diskstream_display ()
 {
+	map_frozen ();
+	update_input_display ();
+
 	if (is_audio_track()) {
 
-		map_frozen ();
-
-		update_input_display ();
-
 		if (input_selector) {
 			input_selector->hide_all ();
 		}
@@ -678,9 +675,6 @@ MixerStrip::update_diskstream_display ()
 
 	} else {
 
-		map_frozen ();
-
-		update_input_display ();
 		show_passthru_color ();
 	}
 }
@@ -760,12 +754,14 @@ void
 MixerStrip::input_changed (IOChange change, void *src)
 {
 	Gtkmm2ext::UI::instance()->call_slot (mem_fun(*this, &MixerStrip::update_input_display));
+	set_width(_width, this);
 }
 
 void
 MixerStrip::output_changed (IOChange change, void *src)
 {
 	Gtkmm2ext::UI::instance()->call_slot (mem_fun(*this, &MixerStrip::update_output_display));
+	set_width(_width, this);
 }
 
 
@@ -992,7 +988,9 @@ MixerStrip::build_route_ops_menu ()
 	build_remote_control_menu ();
 	
 	items.push_back (SeparatorElem());
-	items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
+	if (!Profile->get_sae()) {
+              items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
+        }
 
 	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("Remove"), mem_fun(*this, &RouteUI::remove_this_route)));
@@ -1130,7 +1128,7 @@ MixerStrip::map_frozen ()
 {
 	ENSURE_GUI_THREAD (mem_fun(*this, &MixerStrip::map_frozen));
 
-	AudioTrack* at = audio_track();
+        boost::shared_ptr<AudioTrack> at = audio_track();
 
 	if (at) {
 		switch (at->freeze_state()) {
@@ -1143,9 +1141,17 @@ MixerStrip::map_frozen ()
 			pre_redirect_box.set_sensitive (true);
 			post_redirect_box.set_sensitive (true);
 			speed_spinner.set_sensitive (true);
+			// XXX need some way, maybe, to retoggle redirect editors
 			break;
 		}
 	}
+	
+	hide_redirect_editors ();
+}
+
+void
+MixerStrip::hide_redirect_editors ()
+{
 	_route->foreach_redirect (this, &MixerStrip::hide_redirect_editor);
 }
 
@@ -1167,19 +1173,19 @@ MixerStrip::route_active_changed ()
 	if (is_audio_track()) {
 		if (_route->active()) {
 			set_name ("AudioTrackStripBase");
-			gpm.set_meter_strip_name ("AudioTrackStripBase");
+			gpm.set_meter_strip_name ("AudioTrackMetrics");
 		} else {
 			set_name ("AudioTrackStripBaseInactive");
-			gpm.set_meter_strip_name ("AudioTrackStripBaseInactive");
+			gpm.set_meter_strip_name ("AudioTrackMetricsInactive");
 		}
 		gpm.set_fader_name ("AudioTrackFader");
 	} else { // FIXME: assumed audio bus
 		if (_route->active()) {
 			set_name ("AudioBusStripBase");
-			gpm.set_meter_strip_name ("AudioBusStripBase");
+			gpm.set_meter_strip_name ("AudioBusMetrics");
 		} else {
 			set_name ("AudioBusStripBaseInactive");
-			gpm.set_meter_strip_name ("AudioBusStripBaseInactive");
+			gpm.set_meter_strip_name ("AudioBusMetricsInactive");
 		}
 		gpm.set_fader_name ("AudioBusFader");
 	}
@@ -1222,5 +1228,6 @@ MixerStrip::meter_changed (void *src)
 		}
 
 		gpm.setup_meters ();
+		set_width(_width, this);
 }
 
