@@ -16,9 +16,12 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 */
+#include <pango/pangoft2.h> // for fontmap resolution control for GnomeCanvas
+#include <pango/pangocairo.h> // for fontmap resolution control for GnomeCanvas
 
 #include <pbd/whitespace.h>
 
+#include <ardour/ardour.h>
 #include <ardour/session.h>
 #include <ardour/audioengine.h>
 #include <ardour/configuration.h>
@@ -43,6 +46,7 @@
 #include "option_editor.h"
 #include "midi_port_dialog.h"
 #include "gui_thread.h"
+#include "utils.h"
 
 #include "i18n.h"
 
@@ -100,7 +104,7 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 
 	  /* kbd/mouse */
 
-	  keyboard_mouse_table (3, 4),
+	  keyboard_mouse_table (4, 4),
 	  delete_button_adjustment (3, 1, 5),
 	  delete_button_spin (delete_button_adjustment),
 	  edit_button_adjustment (3, 1, 5),
@@ -249,12 +253,40 @@ OptionEditor::add_session_paths ()
 	session_raid_entry.set_text(session->raid_path());
 }
 
+static void
+font_scale_changed (Gtk::Adjustment* adj)
+{
+	Config->set_font_scale((long)floor (adj->get_value() * 1024));
+	reset_dpi();
+}
+
 void
 OptionEditor::setup_misc_options ()
 {
 	Gtk::HBox* hbox;
-	
-	Label* label = manage (new Label (_("Short crossfade length (msecs)")));
+	Label* label;
+
+#ifndef GTKOSX
+	/* font scaling does nothing with GDK/Quartz */
+
+	Gtk::Adjustment* dpi_adj = new Gtk::Adjustment ((double)Config->get_font_scale() / 1024, 50, 250, 1, 10);
+	Gtk::HScale * dpi_range = new Gtk::HScale (*dpi_adj);
+
+	label = manage (new Label (_("Font Scaling")));
+	label->set_name ("OptionsLabel");
+
+	dpi_range->set_update_policy (Gtk::UPDATE_DISCONTINUOUS);
+	dpi_adj->signal_value_changed().connect (bind (sigc::ptr_fun (font_scale_changed), dpi_adj));
+
+	hbox = manage (new HBox);
+	hbox->set_border_width (5);
+	hbox->set_spacing (10);
+	hbox->pack_start (*label, false, false);
+	hbox->pack_start (*dpi_range, true, true);
+	misc_packer.pack_start (*hbox, false, false);
+#endif
+
+	label = manage (new Label (_("Short crossfade length (msecs)")));
 	label->set_name ("OptionsLabel");
 	
 	hbox = manage (new HBox);
@@ -1196,6 +1228,39 @@ OptionEditor::setup_keyboard_options ()
 	
 	keyboard_mouse_table.attach (*label, 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, FILL);
 	keyboard_mouse_table.attach (snap_modifier_combo, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, FILL);
+
+	vector<string> strs;
+	
+	for (std::map<std::string,std::string>::iterator bf = Keyboard::binding_files.begin(); bf != Keyboard::binding_files.end(); ++bf) {
+		strs.push_back (bf->first);
+	}
+	
+	set_popdown_strings (keyboard_layout_selector, strs);
+	keyboard_layout_selector.set_active_text (Keyboard::current_binding_name());
+	keyboard_layout_selector.signal_changed().connect (mem_fun (*this, &OptionEditor::bindings_changed));
+
+	label = manage (new Label (_("Keyboard layout")));
+	label->set_name ("OptionsLabel");
+	label->set_alignment (1.0, 0.5);
+
+	keyboard_mouse_table.attach (*label, 0, 1, 3, 4, Gtk::FILL|Gtk::EXPAND, FILL);
+	keyboard_mouse_table.attach (keyboard_layout_selector, 1, 2, 3, 4, Gtk::FILL|Gtk::EXPAND, FILL);
+}
+
+void
+OptionEditor::bindings_changed ()
+{
+	string txt;
+	
+	txt = keyboard_layout_selector.get_active_text();
+
+	for (std::map<string,string>::iterator i = Keyboard::binding_files.begin(); i != Keyboard::binding_files.end(); ++i) {
+		if (txt == i->first) {
+			if (Keyboard::load_keybindings (i->second)) {
+				Keyboard::save_keybindings ();
+			}
+		}
+	}
 }
 
 void
@@ -1310,5 +1375,7 @@ OptionEditor::parameter_changed (const char* parameter_name)
 
 		save_history_button.set_active (x);
 		saved_history_depth_spinner.set_sensitive (x);
+	} else if (PARAM_IS ("font-scale")) {
+		reset_dpi();
 	}
 }
