@@ -77,16 +77,19 @@ class PlugUIBase : public virtual sigc::trackable
 	virtual void activate () {}
 	virtual void deactivate () {}
 
+	virtual void update_presets ();
+
   protected:
 	boost::shared_ptr<ARDOUR::PluginInsert> insert;
 	boost::shared_ptr<ARDOUR::Plugin> plugin;
-	Gtk::ComboBoxText combo;
+	Gtk::ComboBoxText preset_combo;
 	Gtk::Button save_button;
 	Gtk::ToggleButton bypass_button;
 
 	void setting_selected();
 	void save_plugin_setting (void);
 	void bypass_toggled();
+	void redirect_active_changed (ARDOUR::Redirect* r, void* src);
 };
 
 class GenericPluginUI : public PlugUIBase, public Gtk::VBox 
@@ -196,24 +199,29 @@ class GenericPluginUI : public PlugUIBase, public Gtk::VBox
 	void print_parameter (char *buf, uint32_t len, uint32_t param);
 };
 
-class PluginUIWindow : public ArdourDialog
+class PluginUIWindow : public Gtk::Window
 {
   public:
-	PluginUIWindow (boost::shared_ptr<ARDOUR::PluginInsert> insert, bool scrollable=false);
+	PluginUIWindow (Gtk::Window*, boost::shared_ptr<ARDOUR::PluginInsert> insert, bool scrollable=false);
 	~PluginUIWindow ();
 
 	PlugUIBase& pluginui() { return *_pluginui; }
 
 	void resize_preferred();
+	void set_parent (Gtk::Window*);
 
 	bool on_key_press_event (GdkEventKey*);
 	bool on_key_release_event (GdkEventKey*);
 	void on_show ();
 	void on_hide ();
-
+	void on_map ();
   private:
 	PlugUIBase* _pluginui;
+	Gtk::Window* parent;
+	Gtk::VBox vbox;
 	bool non_gtk_gui;
+	bool was_visible;
+
 	void app_activated (bool);
 	void plugin_going_away ();
 
