@@ -47,6 +47,7 @@
 #include <ardour/ladspa_plugin.h>
 #include <ardour/connection.h>
 #include <ardour/session_connection.h>
+#include <ardour/profile.h>
 
 #include "ardour_ui.h"
 #include "ardour_dialog.h"
@@ -402,6 +403,9 @@ RedirectBox::insert_plugin_chosen (boost::shared_ptr<Plugin> plugin)
 		if (_route->add_redirect (redirect, this, &err_streams)) {
 			weird_plugin_dialog (*plugin, err_streams, _route);
 		} else {
+			if (Profile->get_sae()) {
+				redirect->set_active (true, 0);
+			}
 			redirect->active_changed.connect (bind (mem_fun (*this, &RedirectBox::show_redirect_active_r), boost::weak_ptr<Redirect>(redirect)));
 		}
 	}
@@ -1084,16 +1088,15 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 		if ((plugin_insert = boost::dynamic_pointer_cast<PluginInsert> (insert)) != 0) {
 			
 			PluginUIWindow *plugin_ui;
+
+			/* these are both allowed to be null */
+			
+			Container* toplevel = get_toplevel();
+			Window* win = dynamic_cast<Gtk::Window*>(toplevel);
 			
 			if (plugin_insert->get_gui() == 0) {
-				
-				plugin_ui = new PluginUIWindow (plugin_insert);
-				
-				if (_owner_is_mixer) {
-					ARDOUR_UI::instance()->the_mixer()->ensure_float (*plugin_ui);
-				} else {
-					ARDOUR_UI::instance()->the_editor().ensure_float (*plugin_ui);
-				}
+
+				plugin_ui = new PluginUIWindow (win, plugin_insert);
 				
 				WindowTitle title(Glib::get_application_name());
 				title += generate_redirect_title (plugin_insert);
@@ -1104,9 +1107,9 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 				// change window title when route name is changed
 				_route->name_changed.connect (bind (mem_fun(*this, &RedirectBox::route_name_changed), plugin_ui, boost::weak_ptr<PluginInsert> (plugin_insert)));
 				
-				
 			} else {
 				plugin_ui = reinterpret_cast<PluginUIWindow *> (plugin_insert->get_gui());
+				plugin_ui->set_parent (win);
 			}
 			
 			if (plugin_ui->is_visible()) {
@@ -1379,7 +1382,7 @@ RedirectBox::route_name_changed (void* src, PluginUIWindow* plugin_ui, boost::we
 string 
 RedirectBox::generate_redirect_title (boost::shared_ptr<PluginInsert> pi)
 {
-	string maker = pi->plugin()->maker();
+	string maker = pi->plugin()->maker() ? pi->plugin()->maker() : "";
 	string::size_type email_pos;
 
 	if ((email_pos = maker.find_first_of ('<')) != string::npos) {
