@@ -93,6 +93,23 @@ RegionView::RegionView (const RegionView& other)
 	_height = other._height;
 }
 
+RegionView::RegionView (const RegionView& other, boost::shared_ptr<Region> other_region)
+	: TimeAxisViewItem (other)
+{
+	/* this is a pseudo-copy constructor used when dragging regions 
+	   around on the canvas.
+	*/
+
+	/* derived concrete type will call init () */
+
+	_region = other_region;
+	editor = other.editor;
+	current_visible_sync_position = other.current_visible_sync_position;
+	valid = false;
+	_pixel_width = other._pixel_width;
+	_height = other._height;
+}
+
 RegionView::RegionView (ArdourCanvas::Group*         parent, 
                         TimeAxisView&                tv,
                         boost::shared_ptr<ARDOUR::Region> r,
@@ -143,7 +160,7 @@ RegionView::init (Gdk::Color& basic_color, bool wfd)
 	set_colors ();
 
 	ColorsChanged.connect (mem_fun (*this, &RegionView::color_handler));
-
+	// set_pango_fontsize();
 	/* XXX sync mark drag? */
 }
 
