@@ -43,6 +43,7 @@
 #include "public_editor.h"
 #include "time_axis_view.h"
 #include "simplerect.h"
+#include "simpleline.h"
 #include "selection.h"
 #include "keyboard.h"
 #include "rgb_macros.h"
@@ -71,7 +72,7 @@ bool TimeAxisView::need_size_info = true;
 TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisView* rent, Canvas& canvas) 
 	: AxisView (sess), 
 	  editor (ed),
-	  controls_table (2, 7)
+	  controls_table (2, 8)
 {
 	if (need_size_info) {
 		compute_controls_size_info ();
@@ -125,7 +126,7 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_table.set_col_spacings (0);
 	controls_table.set_homogeneous (true);
 
-	controls_table.attach (name_hbox, 0, 4, 0, 1, Gtk::SHRINK|Gtk::EXPAND, Gtk::SHRINK|Gtk::EXPAND);
+	controls_table.attach (name_hbox, 0, 5, 0, 1,  Gtk::FILL|Gtk::EXPAND,  Gtk::FILL|Gtk::EXPAND, 3, 0);
 	controls_table.show_all ();
 	controls_table.set_no_show_all ();
 
@@ -140,13 +141,12 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_ebox.signal_button_release_event().connect (mem_fun (*this, &TimeAxisView::controls_ebox_button_release));
 	controls_ebox.signal_scroll_event().connect (mem_fun (*this, &TimeAxisView::controls_ebox_scroll), true);
 
-	controls_lhs_pad.set_name ("TimeAxisViewControlsPadding");
-	controls_hbox.pack_start (controls_lhs_pad,false,false);
 	controls_hbox.pack_start (controls_ebox,true,true);
 	controls_hbox.show ();
 
 	controls_frame.add (controls_hbox);
 	controls_frame.set_name ("TimeAxisViewControlsBaseUnselected");
+	controls_vbox.set_name ("TimeAxisViewControlsBaseUnselected");
 	controls_frame.set_shadow_type (Gtk::SHADOW_ETCHED_OUT);
 
 	ColorsChanged.connect (mem_fun (*this, &TimeAxisView::color_handler));
@@ -171,6 +171,10 @@ TimeAxisView::~TimeAxisView()
 		delete (*i)->end_trim;
 	}
 
+	for (list<SimpleLine*>::iterator i = feature_lines.begin(); i != feature_lines.end(); ++i) {
+		delete (*i);
+	}
+
 	if (selection_group) {
 		delete selection_group;
 		selection_group = 0;
@@ -353,6 +357,16 @@ TimeAxisView::step_height (bool bigger)
        }
 }
 
+void
+TimeAxisView::set_heights (TrackHeight h)
+{
+	TrackSelection& ts (editor.get_selection().tracks);
+
+	for (TrackSelection::iterator i = ts.begin(); i != ts.end(); ++i) {
+		(*i)->set_height (h);
+	}
+}
+
 void
 TimeAxisView::set_height (TrackHeight h)
 {
@@ -513,6 +527,18 @@ TimeAxisView::name_entry_button_release (GdkEventButton *ev)
 	return false;
 }
 
+void
+TimeAxisView::conditionally_add_to_selection ()
+{
+	Selection& s (editor.get_selection());
+
+	if (!s.selected (this)) {
+		cerr << "set selected track\n";
+		editor.set_selected_track (*this, Selection::Set);
+	}
+}
+
+
 void
 TimeAxisView::popup_display_menu (guint32 when)
 {
@@ -520,14 +546,14 @@ TimeAxisView::popup_display_menu (guint32 when)
 		build_display_menu ();
 	}
 
-	editor.set_selected_track (*this, Selection::Add);
+	conditionally_add_to_selection ();
 	display_menu->popup (1, when);	
 }
 
 gint
 TimeAxisView::size_click (GdkEventButton *ev)
 {
-	editor.set_selected_track (*this, Selection::Add);
+	conditionally_add_to_selection ();
 	popup_size_menu (ev->time);
 	return TRUE;
 }
@@ -553,7 +579,7 @@ TimeAxisView::set_selected (bool yn)
 	if (_selected) {
 		controls_ebox.set_name (controls_base_selected_name);
 		controls_frame.set_name (controls_base_selected_name);
-		
+		controls_vbox.set_name (controls_base_selected_name);
 		/* propagate any existing selection, if the mode is right */
 
 		if (editor.current_mouse_mode() == Editing::MouseRange && !editor.get_selection().time.empty()) {
@@ -563,7 +589,7 @@ TimeAxisView::set_selected (bool yn)
 	} else {
 		controls_ebox.set_name (controls_base_unselected_name);
 		controls_frame.set_name (controls_base_unselected_name);
-
+		controls_vbox.set_name (controls_base_unselected_name);
 		hide_selection ();
 
 		/* children will be set for the yn=true case. but when deselecting
@@ -586,12 +612,12 @@ TimeAxisView::build_size_menu ()
 	size_menu->set_name ("ArdourContextMenu");
 	MenuList& items = size_menu->items();
 	
-	items.push_back (MenuElem (_("Largest"), bind (mem_fun (*this, &TimeAxisView::set_height), Largest)));
-	items.push_back (MenuElem (_("Large"), bind (mem_fun (*this, &TimeAxisView::set_height), Large)));
-	items.push_back (MenuElem (_("Larger"), bind (mem_fun (*this, &TimeAxisView::set_height), Larger)));
-	items.push_back (MenuElem (_("Normal"), bind (mem_fun (*this, &TimeAxisView::set_height), Normal)));
-	items.push_back (MenuElem (_("Smaller"), bind (mem_fun (*this, &TimeAxisView::set_height),Smaller)));
-	items.push_back (MenuElem (_("Small"), bind (mem_fun (*this, &TimeAxisView::set_height), Small)));
+	items.push_back (MenuElem (_("Largest"), bind (mem_fun (*this, &TimeAxisView::set_heights), Largest)));
+	items.push_back (MenuElem (_("Large"), bind (mem_fun (*this, &TimeAxisView::set_heights), Large)));
+	items.push_back (MenuElem (_("Larger"), bind (mem_fun (*this, &TimeAxisView::set_heights), Larger)));
+	items.push_back (MenuElem (_("Normal"), bind (mem_fun (*this, &TimeAxisView::set_heights), Normal)));
+	items.push_back (MenuElem (_("Smaller"), bind (mem_fun (*this, &TimeAxisView::set_heights),Smaller)));
+	items.push_back (MenuElem (_("Small"), bind (mem_fun (*this, &TimeAxisView::set_heights), Small)));
 }
 
 void
@@ -608,6 +634,14 @@ TimeAxisView::build_display_menu ()
 void
 TimeAxisView::set_samples_per_unit (double spu)
 {
+	AnalysisFeatureList::const_iterator i;
+	list<ArdourCanvas::SimpleLine*>::iterator l;
+
+	for (i = analysis_features.begin(), l = feature_lines.begin(); i != analysis_features.end() && l != feature_lines.end(); ++i, ++l) {
+		(*l)->property_x1() = editor.frame_to_pixel (*i);
+		(*l)->property_x2() = editor.frame_to_pixel (*i);
+	}
+	
 	for (vector<TimeAxisView*>::iterator i = children.begin(); i != children.end(); ++i) {
 		(*i)->set_samples_per_unit (spu);
 	}
@@ -1101,3 +1135,49 @@ TimeAxisView::covers_y_position (double y)
 
 	return 0;
 }
+
+void
+TimeAxisView::show_feature_lines (const AnalysisFeatureList& pos)
+{
+	analysis_features = pos;
+	reshow_feature_lines ();
+}
+
+
+void
+TimeAxisView::hide_feature_lines ()
+{
+	list<ArdourCanvas::SimpleLine*>::iterator l;
+
+	for (l = feature_lines.begin(); l != feature_lines.end(); ++l) {
+		(*l)->hide();
+	}
+}
+
+void
+TimeAxisView::reshow_feature_lines ()
+{
+	while (feature_lines.size()< analysis_features.size()) {
+		ArdourCanvas::SimpleLine* l = new ArdourCanvas::SimpleLine (*canvas_display);
+		l->property_color_rgba() = (guint) ARDOUR_UI::config()->canvasvar_ZeroLine.get();
+		l->property_y1() = 0;
+		l->property_y2() = height;
+		feature_lines.push_back (l);
+	}
+
+	while (feature_lines.size() > analysis_features.size()) {
+		ArdourCanvas::SimpleLine *line = feature_lines.back();
+		feature_lines.pop_back ();
+		delete line;
+	}
+
+	AnalysisFeatureList::const_iterator i;
+	list<ArdourCanvas::SimpleLine*>::iterator l;
+
+	for (i = analysis_features.begin(), l = feature_lines.begin(); i != analysis_features.end() && l != feature_lines.end(); ++i, ++l) {
+		(*l)->property_x1() = editor.frame_to_pixel (*i);
+		(*l)->property_x2() = editor.frame_to_pixel (*i);
+		(*l)->show ();
+	}
+}
+
