@@ -67,6 +67,8 @@ Pango::FontDescription* get_font_for_style (std::string widgetname);
 
 uint32_t rgba_from_style (std::string, uint32_t, uint32_t, uint32_t, uint32_t, std::string = "fg", int = Gtk::STATE_NORMAL, bool = true);
 
+Gdk::Color color_from_style (std::string widget_style_name, int state, std::string attr);
+
 void decorate (Gtk::Window& w, Gdk::WMDecoration d);
 
 bool canvas_item_visible (ArdourCanvas::Item* item);
@@ -81,5 +83,7 @@ static std::map<std::string, Glib::RefPtr<Gdk::Pixbuf> > xpm_map;
 const char* const *get_xpm_data (std::string path);
 std::string longest (std::vector<std::string>&);
 bool key_is_legal_for_numeric_entry (guint keyval);
+void reset_dpi ();
+void set_pango_fontsize ();
 
 #endif /* __ardour_gtk_utils_h__ */
