@@ -32,6 +32,7 @@
 #include <ardour/plugin.h>
 
 #include <AudioUnit/AudioUnit.h>
+#include <appleutility/AUParamInfo.h>
 
 #include <boost/shared_ptr.hpp>
 
@@ -45,10 +46,21 @@ namespace ARDOUR {
 class AudioEngine;
 class Session;
 
+struct AUParameterDescriptor : public Plugin::ParameterDescriptor {
+	// additional fields to make operations more efficient
+	AudioUnitParameterID id;
+	AudioUnitScope scope;
+	AudioUnitElement element;
+	float default_value;
+	bool automatable;
+	AudioUnitParameterUnit unit;
+};
+
 class AUPlugin : public ARDOUR::Plugin
 {
   public:
 	AUPlugin (AudioEngine& engine, Session& session, boost::shared_ptr<CAComponent> comp);
+	AUPlugin (const AUPlugin& other);
 	virtual ~AUPlugin ();
 	
         std::string unique_id () const;
@@ -94,7 +106,7 @@ class AUPlugin : public ARDOUR::Plugin
 	uint32_t input_streams() const;
 
 	boost::shared_ptr<CAAudioUnit> get_au () { return unit; }
-	boost::shared_ptr<CAComponent> get_comp () { return comp; }
+	boost::shared_ptr<CAComponent> get_comp () const { return comp; }
     
         OSStatus render_callback(AudioUnitRenderActionFlags *ioActionFlags,
 				 const AudioTimeStamp    *inTimeStamp,
@@ -104,7 +116,7 @@ class AUPlugin : public ARDOUR::Plugin
   private:
         boost::shared_ptr<CAComponent> comp;
         boost::shared_ptr<CAAudioUnit> unit;
-	
+
 	AudioStreamBasicDescription streamFormat;
         bool initialized;
         int format_set;
@@ -118,6 +130,7 @@ class AUPlugin : public ARDOUR::Plugin
 	int set_input_format ();
 	int set_stream_format (int scope, uint32_t cnt);
         int _set_block_size (nframes_t nframes);
+	void discover_parameters ();
 
 	std::vector<std::pair<uint32_t, uint32_t> > parameter_map;
 	uint32_t current_maxbuf;
@@ -125,6 +138,9 @@ class AUPlugin : public ARDOUR::Plugin
         nframes_t cb_offset;
         vector<Sample*>* current_buffers;
         nframes_t frames_processed;
+	
+	std::vector<AUParameterDescriptor> descriptors;
+	void init ();
 };
 	
 typedef boost::shared_ptr<AUPlugin> AUPluginPtr;
