@@ -162,6 +162,7 @@ class Session : public PBD::StatefulDestructible
 	    union {
 			void*                ptr;
 			bool                 yes_or_no;
+		        nframes_t            target2_frame;
 			SlaveSource slave;
 	    };
 
@@ -248,10 +249,18 @@ class Session : public PBD::StatefulDestructible
 	bool deletion_in_progress() const { return _state_of_the_state & Deletion; }
 	sigc::signal<void> DirtyChanged;
 
+	static sigc::signal<void> AutoBindingOn;
+	static sigc::signal<void> AutoBindingOff;
+
+	static sigc::signal<void,std::string> Dialog;
+	
 	std::string sound_dir (bool with_path = true) const;
 	std::string peak_dir () const;
 	std::string dead_sound_dir () const;
 	std::string automation_dir () const;
+	std::string analysis_dir() const;
+	
+	int ensure_subdirs ();
 
 	Glib::ustring peak_path (Glib::ustring) const;
 
@@ -343,7 +352,7 @@ class Session : public PBD::StatefulDestructible
 	sigc::signal<void> TransportStateChange; /* generic */
 	sigc::signal<void,nframes_t> PositionChanged; /* sent after any non-sequential motion */
 	sigc::signal<void> DurationChanged;
-	sigc::signal<void> HaltOnXrun;
+	sigc::signal<void,nframes_t> Xrun;
 	sigc::signal<void> TransportLooped;
 
 	sigc::signal<void,RouteList&> RouteAdded;
@@ -378,7 +387,10 @@ class Session : public PBD::StatefulDestructible
 	nframes_t get_maximum_extent () const;
 	nframes_t current_end_frame() const { return end_location->start(); }
 	nframes_t current_start_frame() const { return start_location->start(); }
+	// "actual" sample rate of session, set by current audioengine rate, pullup/down etc.
 	nframes_t frame_rate() const   { return _current_frame_rate; }
+	// "native" sample rate of session, regardless of current audioengine rate, pullup/down etc
+	nframes_t nominal_frame_rate() const   { return _nominal_frame_rate; }
 	nframes_t frames_per_hour() const { return _frames_per_hour; }
 
 	double frames_per_smpte_frame() const { return _frames_per_smpte_frame; }
@@ -548,7 +560,10 @@ class Session : public PBD::StatefulDestructible
 	
 	/* region info  */
 
+	void add_regions (std::vector<boost::shared_ptr<Region> >&);
+
 	sigc::signal<void,boost::weak_ptr<AudioRegion> > AudioRegionAdded;
+	sigc::signal<void,std::vector<boost::weak_ptr<AudioRegion> >& > AudioRegionsAdded;
 	sigc::signal<void,boost::weak_ptr<AudioRegion> > AudioRegionRemoved;
 
 	int region_name (string& result, string base = string(""), bool newlevel = false) const;
@@ -568,10 +583,10 @@ class Session : public PBD::StatefulDestructible
 	    string doing_what;
 	    
 	    /* control info */
-	    bool sample_convert;
 	    SrcQuality quality;
 	    volatile bool freeze;
 	    std::vector<Glib::ustring> paths;
+	    bool replace_existing_source;
 	    
 	    /* result */
 	    SourceList sources;
@@ -612,6 +627,12 @@ class Session : public PBD::StatefulDestructible
 	
 	sigc::signal<int,boost::shared_ptr<ARDOUR::Playlist> > AskAboutPlaylistDeletion;
 
+	/* handlers should return 0 for "ignore the rate mismatch"
+	   and !0 for "do not use this session"
+	*/
+
+	static sigc::signal<int,nframes_t, nframes_t> AskAboutSampleRateMismatch;
+
 	/* handlers should return !0 for use pending state, 0 for
 	   ignore it.
 	*/
@@ -980,6 +1001,7 @@ class Session : public PBD::StatefulDestructible
 	bool                     waiting_for_sync_offset;
 	nframes_t               _base_frame_rate;
 	nframes_t               _current_frame_rate;  //this includes video pullup offset
+	nframes_t               _nominal_frame_rate;  //ignores audioengine setting, "native" SR
 	int                      transport_sub_state;
 	mutable gint            _record_status;
 	volatile nframes_t      _transport_frame;
@@ -1478,6 +1500,12 @@ class Session : public PBD::StatefulDestructible
 
 	AudioSourceList audio_sources;
 
+  public:
+	AudioSourceList get_audio_sources() { return audio_sources; }
+    
+  private:
+
+
 	int load_sources (const XMLNode& node);
 	XMLNode& get_sources_as_xml ();
 
