@@ -1598,7 +1598,9 @@ AudioDiskstream::transport_stopped (struct tm& when, time_t twhen, bool abort_ca
 			s->update_header (capture_info.front()->start, when, twhen);
 			s->set_captured_for (_name);
 			s->mark_immutable ();
-			Analyser::queue_source_for_analysis (s, true);
+			if (Config->get_auto_analyse_audio()) {
+				Analyser::queue_source_for_analysis (s, true);
+			}
 		}
 	}
 
@@ -1862,8 +1864,8 @@ AudioDiskstream::disengage_record_enable ()
 {
 	g_atomic_int_set (&_record_enabled, 0);
 	boost::shared_ptr<ChannelList> c = channels.reader();
-	for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
-		if (Config->get_monitoring_model() == HardwareMonitoring) {
+	if (Config->get_monitoring_model() == HardwareMonitoring) {
+		for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
 			if ((*chan)->source) {
 				(*chan)->source->ensure_monitor_input (false);
 			}
@@ -2243,7 +2245,7 @@ AudioDiskstream::add_channel (uint32_t how_many)
 int
 AudioDiskstream::remove_channel_from (boost::shared_ptr<ChannelList> c, uint32_t how_many)
 {
-	while (--how_many && !c->empty()) {
+	while (how_many-- && !c->empty()) {
 		delete c->back();
 		c->pop_back();
 	}
