@@ -138,14 +138,22 @@ AudioTrack::deprecated_use_diskstream_connections ()
 		
 		if (c == 0) {
 		  	error << string_compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
+
+			string replacement_connection;
+
+			if (prop->value().find ('+') != string::npos) {
+				replacement_connection = _("in 1+2");
+			} else {
+				replacement_connection = _("in 1");
+			}
 			
-			if ((c = _session.connection_by_name (_("in 1"))) == 0) {
+			if ((c = _session.connection_by_name (replacement_connection)) == 0) {
 			  	error << _("No input connections available as a replacement")
 			        << endmsg;
 				return -1;
 			} else {
-			  	info << string_compose (_("Connection %1 was not available - \"in 1\" used instead"), prop->value())
-			       << endmsg;
+			  	info << string_compose (_("Connection %1 was not available - \"%2\" used instead"), prop->value(), replacement_connection)
+				     << endmsg;
 			}
 		}
 
@@ -469,19 +477,24 @@ AudioTrack::no_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_fra
 		send_silence = true;
 	} else {
 
-		if (Config->get_auto_input()) {
-			if (Config->get_monitoring_model() == SoftwareMonitoring) {
+		if (!Config->get_tape_machine_mode()) {
+			/* 
+			   ADATs work in a strange way.. 
+			   they monitor input always when stopped.and auto-input is engaged. 
+			*/
+			if ((Config->get_monitoring_model() == SoftwareMonitoring) && (Config->get_auto_input () || _diskstream->record_enabled())) {
 				send_silence = false;
 			} else {
 				send_silence = true;
 			}
 		} else {
-			if (_diskstream->record_enabled()) {
-				if (Config->get_monitoring_model() == SoftwareMonitoring) {
-					send_silence = false;
-				} else {
-					send_silence = true;
-				}
+			/* 
+			   Other machines switch to input on stop if the track is record enabled,
+			   regardless of the auto input setting (auto input only changes the 
+			   monitoring state when the transport is rolling) 
+			*/
+			if ((Config->get_monitoring_model() == SoftwareMonitoring) && _diskstream->record_enabled()) {
+				send_silence = false;
 			} else {
 				send_silence = true;
 			}
@@ -531,7 +544,7 @@ AudioTrack::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame,
 		if (lm.locked()) {
 			// automation snapshot can also be called from the non-rt context
 			// and it uses the redirect list, so we take the lock out here
-			automation_snapshot (start_frame);
+			automation_snapshot (start_frame, false);
 		}
 	}
 
