@@ -77,9 +77,6 @@ AudioEngine::AudioEngine (string client_name)
 	if (connect_to_jack (client_name)) {
 		throw NoBackendAvailable ();
 	}
-
-	start_metering_thread();
-
 }
 
 AudioEngine::~AudioEngine ()
@@ -114,6 +111,8 @@ AudioEngine::start ()
 		if (session) {
 			nframes_t blocksize = jack_get_buffer_size (_jack);
 
+			BootMessage (_("Connect session to engine"));
+
 			session->set_block_size (blocksize);
 			session->set_frame_rate (jack_get_sample_rate (_jack));
 
@@ -155,6 +154,8 @@ AudioEngine::start ()
 		} else {
 			// error << _("cannot activate JACK client") << endmsg;
 		}
+
+		start_metering_thread();
 	}
 
 	return _running ? 0 : -1;
@@ -165,11 +166,11 @@ AudioEngine::stop (bool forever)
 {
 	if (_running) {
 		_running = false;
+		stop_metering_thread ();
 		if (forever) {
 			jack_client_t* foo = _jack;
 			_jack = 0;
 			jack_client_close (foo);
-			stop_metering_thread ();
 		} else {
 			jack_deactivate (_jack);
 		}
@@ -419,8 +420,9 @@ void
 AudioEngine::start_metering_thread ()
 {
 	if (m_meter_thread == 0) {
+		g_atomic_int_set (&m_meter_exit, 0);
 		m_meter_thread = Glib::Thread::create (sigc::mem_fun(this, &AudioEngine::meter_thread),
-				500000, true, true, Glib::THREAD_PRIORITY_NORMAL);
+						       500000, true, true, Glib::THREAD_PRIORITY_NORMAL);
 	}
 }
 
@@ -799,6 +801,8 @@ AudioEngine::halted (void *arg)
 	AudioEngine* ae = static_cast<AudioEngine *> (arg);
 	bool was_running = ae->_running;
 
+	ae->stop_metering_thread ();
+
 	ae->_running = false;
 	ae->_buffer_size = 0;
 	ae->_frame_rate = 0;
@@ -809,6 +813,25 @@ AudioEngine::halted (void *arg)
 	}
 }
 
+bool
+AudioEngine::can_request_hardware_monitoring () 
+{
+	const char ** ports;
+
+	if (!_jack) {
+		return 0;
+	}
+
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortCanMonitor)) == 0) {
+		return false;
+	}
+
+	free (ports);
+
+	return true;
+}
+
+
 uint32_t
 AudioEngine::n_physical_audio_outputs () const
 {
@@ -823,10 +846,9 @@ AudioEngine::n_physical_audio_outputs () const
 		return 0;
 	}
 
-	if (ports) {
-		for (i = 0; ports[i]; ++i);
-		free (ports);
-	}
+	for (i = 0; ports[i]; ++i);
+	free (ports);
+
 	return i;
 }
 
@@ -1108,6 +1130,7 @@ AudioEngine::disconnect_from_jack ()
 	_frame_rate = 0;
 
 	if (_running) {
+		stop_metering_thread ();
 		_running = false;
 		Stopped(); /* EMIT SIGNAL */
 	}
@@ -1212,6 +1235,8 @@ AudioEngine::reconnect_to_jack ()
 
 	Running (); /* EMIT SIGNAL*/
 
+	start_metering_thread ();
+
 	return 0;
 }
 
