@@ -39,6 +39,7 @@
 
 #include <glibmm/miscutils.h>
 #include <glibmm/fileutils.h>
+#include <glibmm/thread.h>
 
 #include <ardour/audiofilesource.h>
 #include <ardour/sndfile_helpers.h>
@@ -69,6 +70,21 @@ uint64_t           AudioFileSource::header_position_offset = 0;
 /* XXX maybe this too */
 char   AudioFileSource::bwf_serial_number[13] = "000000000000";
 
+struct SizedSampleBuffer {
+    nframes_t size;
+    Sample* buf;
+
+    SizedSampleBuffer (nframes_t sz) : size (sz) { 
+	    buf = new Sample[size];
+    }
+
+    ~SizedSampleBuffer() {
+	    delete [] buf;
+    }
+};
+
+Glib::StaticPrivate<SizedSampleBuffer> thread_interleave_buffer = GLIBMM_STATIC_PRIVATE_INIT;
+
 AudioFileSource::AudioFileSource (Session& s, ustring path, Flag flags)
 	: AudioSource (s, path), _flags (flags),
 	  _channel (0)
@@ -680,27 +696,39 @@ AudioFileSource::setup_peakfile ()
 bool
 AudioFileSource::safe_file_extension(ustring file)
 {
-	return !(file.rfind(".wav") == ustring::npos &&
-		file.rfind(".aiff")== ustring::npos &&
-		file.rfind(".aif") == ustring::npos &&
-		file.rfind(".snd") == ustring::npos &&
-		file.rfind(".au")  == ustring::npos &&
-		file.rfind(".raw") == ustring::npos &&
-		file.rfind(".sf")  == ustring::npos &&
-		file.rfind(".cdr") == ustring::npos &&
-		file.rfind(".smp") == ustring::npos &&
-		file.rfind(".maud")== ustring::npos &&
-		file.rfind(".vwe") == ustring::npos &&
-		file.rfind(".paf") == ustring::npos &&
+	const char* suffixes[] = {
+		".wav", ".WAV",
+		".aiff", ".AIFF",
+		".caf", ".CAF",
+		".aif", ".AIF",
+		".amb", ".AMB",
+		".snd", ".SND",
+		".au", ".AU",
+		".raw", ".RAW",
+		".sf", ".SF",
+		".cdr", ".CDR",
+		".smp", ".SMP",
+		".maud", ".MAUD",
+		".vwe", ".VWE",
+		".paf",
 #ifdef HAVE_FLAC
-		file.rfind(".flac")== ustring::npos &&
+		".flac", ".FLAC",
 #endif // HAVE_FLAC
 #ifdef HAVE_COREAUDIO
-		file.rfind(".mp3") == ustring::npos &&
-		file.rfind(".aac") == ustring::npos &&
-		file.rfind(".mp4") == ustring::npos &&
+		".mp3", ".MP3",
+		".aac", ".AAC",
+		".mp4", ".MP4",
 #endif // HAVE_COREAUDIO
-		file.rfind(".voc") == ustring::npos);
+		".voc", ".VOC"
+	};
+
+	for (size_t n = 0; n < sizeof(suffixes)/sizeof(suffixes[0]); ++n) {
+		if (file.rfind (suffixes[n]) == file.length() - strlen (suffixes[n])) {
+			return true;
+		}
+	}
+
+	return false;
 }
 
 void
@@ -714,3 +742,22 @@ AudioFileSource::mark_immutable ()
 		_flags = Flag (_flags & ~(Writable|Removable|RemovableIfEmpty|RemoveAtDestroy|CanRename));
 	}
 }
+
+
+Sample*
+AudioFileSource::get_interleave_buffer (nframes_t size)
+{
+	SizedSampleBuffer* ssb;
+
+	if ((ssb = thread_interleave_buffer.get()) == 0) {
+		ssb = new SizedSampleBuffer (size);
+		thread_interleave_buffer.set (ssb);
+	}
+
+	if (ssb->size < size) {
+		ssb = new SizedSampleBuffer (size);
+		thread_interleave_buffer.set (ssb);
+	}
+
+	return ssb->buf;
+}
