@@ -23,6 +23,7 @@
 
 #include <pbd/failed_constructor.h>
 #include <pbd/xml++.h>
+#include <pbd/stacktrace.h>
 
 #include <ardour/insert.h>
 #include <ardour/plugin.h>
@@ -30,6 +31,10 @@
 #include <ardour/route.h>
 #include <ardour/ladspa_plugin.h>
 
+#ifdef HAVE_SLV2
+#include <ardour/lv2_plugin.h>
+#endif
+
 #ifdef VST_SUPPORT
 #include <ardour/vst_plugin.h>
 #endif
@@ -316,7 +321,7 @@ PluginInsert::connect_and_run (vector<Sample*>& bufs, uint32_t nbufs, nframes_t
 }
 
 void
-PluginInsert::automation_snapshot (nframes_t now)
+PluginInsert::automation_snapshot (nframes_t now, bool force)
 {
 	map<uint32_t,AutomationList*>::iterator li;
 	
@@ -352,12 +357,11 @@ PluginInsert::silence (nframes_t nframes, nframes_t offset)
 {
 	int32_t in_index = 0;
 	int32_t out_index = 0;
-
-	uint32_t n;
+	int32_t n;
 
 	if (active()) {
 		for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
-			n = (*i) -> get_info()->n_inputs;
+			n = input_streams();
 			(*i)->connect_and_run (_session.get_silent_buffers (n), n, in_index, out_index, nframes, offset);
 		}
 	}
@@ -373,21 +377,20 @@ PluginInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nf
 		} else {
 			connect_and_run (bufs, nbufs, nframes, offset, false);
 		}
+
 	} else {
-		uint32_t in = _plugins[0]->get_info()->n_inputs;
-		uint32_t out = _plugins[0]->get_info()->n_outputs;
 
-		if (in < 0 || out < 0) {
+		uint32_t in = input_streams ();
+		uint32_t out = output_streams ();
+
+		if (out > in) {
 			
-		} else {
-
-			if (out > in) {
-				
-				/* not active, but something has make up for any channel count increase */
-				
-				for (uint32_t n = out - in; n < out; ++n) {
-					memcpy (bufs[n], bufs[in - 1], sizeof (Sample) * nframes);
-				}
+			/* not active, but something has make up for any channel count increase,
+			   so copy the last buffer to the extras.
+			*/
+			
+			for (uint32_t n = out - in; n < out && n < nbufs; ++n) {
+				memcpy (bufs[n], bufs[in - 1], sizeof (Sample) * nframes);
 			}
 		}
 	}
@@ -515,6 +518,9 @@ boost::shared_ptr<Plugin>
 PluginInsert::plugin_factory (boost::shared_ptr<Plugin> other)
 {
 	boost::shared_ptr<LadspaPlugin> lp;
+#ifdef HAVE_SLV2
+	boost::shared_ptr<LV2Plugin> lv2p;
+#endif
 #ifdef VST_SUPPORT
 	boost::shared_ptr<VSTPlugin> vp;
 #endif
@@ -524,6 +530,10 @@ PluginInsert::plugin_factory (boost::shared_ptr<Plugin> other)
 
 	if ((lp = boost::dynamic_pointer_cast<LadspaPlugin> (other)) != 0) {
 		return boost::shared_ptr<Plugin> (new LadspaPlugin (*lp));
+#ifdef HAVE_SLV2
+	} else if ((lv2p = boost::dynamic_pointer_cast<LV2Plugin> (other)) != 0) {
+		return boost::shared_ptr<Plugin> (new LV2Plugin (*lv2p));
+#endif
 #ifdef VST_SUPPORT
 	} else if ((vp = boost::dynamic_pointer_cast<VSTPlugin> (other)) != 0) {
 		return boost::shared_ptr<Plugin> (new VSTPlugin (*vp));
@@ -673,6 +683,8 @@ PluginInsert::set_state(const XMLNode& node)
 
 	if (prop->value() == X_("ladspa") || prop->value() == X_("Ladspa")) { /* handle old school sessions */
 		type = ARDOUR::LADSPA;
+	} else if (prop->value() == X_("lv2")) {
+		type = ARDOUR::LV2;
 	} else if (prop->value() == X_("vst")) {
 		type = ARDOUR::VST;
 	} else if (prop->value() == X_("audiounit")) {
@@ -704,7 +716,7 @@ PluginInsert::set_state(const XMLNode& node)
 	}
 
 	boost::shared_ptr<Plugin> plugin;
-	
+
 	plugin = find_plugin (_session, prop->value(), type);	
 
 	if (plugin == 0) {
@@ -913,15 +925,6 @@ PortInsert::PortInsert (const PortInsert& other)
 void
 PortInsert::init ()
 {
-	if (add_input_port ("", this)) {
-		error << _("PortInsert: cannot add input port") << endmsg;
-		throw failed_constructor();
-	}
-	
-	if (add_output_port ("", this)) {
-		error << _("PortInsert: cannot add output port") << endmsg;
-		throw failed_constructor();
-	}
 }
 
 PortInsert::PortInsert (Session& s, const XMLNode& node)
@@ -1054,7 +1057,7 @@ PortInsert::can_support_input_configuration (int32_t in) const
 
 		/* not configured yet */
 
-		return 1; /* we can support anything the first time we're asked */
+		return in; /* we can support anything the first time we're asked */
 
 	} else {
 
@@ -1063,7 +1066,7 @@ PortInsert::can_support_input_configuration (int32_t in) const
 		*/
 
 		if (output_maximum() == in) {
-			return 1;
+			return in;
 		} 
 	}
 
@@ -1077,8 +1080,12 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 	   the last request config. or something like that.
 	*/
 
+	set_output_maximum (in);
+	set_output_minimum (in);
+	set_input_maximum (out);
+	set_input_minimum (out);
 
-	/* this is a bit odd: 
+	/* this can be momentarily confusing: 
 
 	   the number of inputs we are required to handle corresponds 
 	   to the number of output ports we need.
@@ -1087,11 +1094,6 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 	   to the number of input ports we need.
 	*/
 
-	set_output_maximum (in);
-	set_output_minimum (in);
-	set_input_maximum (out);
-	set_input_minimum (out);
-
 	if (in < 0) {
 		in = n_outputs ();
 	} 
@@ -1106,6 +1108,11 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 int32_t
 PortInsert::compute_output_streams (int32_t cnt) const
 {
+	if (n_inputs() == 0) {
+		/* not configured yet */
+		return cnt;
+	}
+
 	/* puzzling, eh? think about it ... */
 	return n_inputs ();
 }
