@@ -32,6 +32,7 @@
 
 #include <ardour/audioengine.h>
 #include <ardour/io.h>
+#include <ardour/route.h>
 #include <ardour/port.h>
 #include <ardour/connection.h>
 #include <ardour/session.h>
@@ -953,7 +954,7 @@ IO::add_input_port (string source, void* src, DataType type)
 		
 		{ 
 			Glib::Mutex::Lock lm (io_lock);
-			
+
 			if (_input_maximum >= 0 && (int) _ninputs == _input_maximum) {
 				return -1;
 			}
@@ -2056,12 +2057,21 @@ IO::parse_gain_string (const string& str, vector<string>& ports)
 }
 
 int
-IO::set_name (string name, void* src)
+IO::set_name (string requested_name, void* src)
 {
-	if (name == _name) {
+	if (requested_name == _name) {
 		return 0;
 	}
 
+	string name;
+	Route *rt;
+	if ( (rt = dynamic_cast<Route *>(this))) {
+		name = Route::ensure_track_or_route_name(requested_name, _session);
+	} else {
+		name = requested_name;
+	}
+
+
 	/* replace all colons in the name. i wish we didn't have to do this */
 
 	if (replace_all (name, ":", "-")) {
@@ -2436,9 +2446,8 @@ IO::setup_peak_meters ()
 void
 IO::update_meters()
 {
-    Glib::Mutex::Lock guard (m_meter_signal_lock);
-    
-    Meter();
+	Glib::Mutex::Lock guard (m_meter_signal_lock);
+	Meter();
 }
 
 void
@@ -2594,18 +2603,14 @@ IO::end_pan_touch (uint32_t which)
 }
 
 void
-IO::automation_snapshot (nframes_t now)
+IO::automation_snapshot (nframes_t now, bool force)
 {
-	if (last_automation_snapshot > now || (now - last_automation_snapshot) > _automation_interval) {
-
-		if (gain_automation_recording()) {
-			_gain_automation_curve.rt_add (now, gain());
-		}
-		
-		_panner->snapshot (now);
-
-		last_automation_snapshot = now;
+	if (gain_automation_recording()) {
+		_gain_automation_curve.rt_add (now, gain());
 	}
+	
+	_panner->snapshot (now);
+	last_automation_snapshot = now;
 }
 
 void
