@@ -46,6 +46,10 @@
 #include <ardour/audio_unit.h>
 #endif
 
+#ifdef HAVE_SLV2
+#include <ardour/lv2_plugin.h>
+#endif
+
 #include <pbd/stl_delete.h>
 
 #include "i18n.h"
@@ -73,9 +77,7 @@ Plugin::setup_controls ()
 	   we'll fill this in on an as-needed basis.
 	*/
 
-	for (uint32_t i = 0; i < port_cnt; ++i) {
-		controls.push_back (0);
-	}
+	controls.assign (port_cnt, (PortControllable*) 0);
 }
 
 Plugin::~Plugin ()
@@ -314,6 +316,12 @@ ARDOUR::find_plugin(Session& session, string identifier, PluginType type)
 	case ARDOUR::LADSPA:
 		plugs = mgr->ladspa_plugin_info();
 		break;
+	
+#ifdef HAVE_SLV2
+	case ARDOUR::LV2:
+		plugs = mgr->lv2_plugin_info();
+		break;
+#endif
 
 #ifdef VST_SUPPORT
 	case ARDOUR::VST:
@@ -338,6 +346,19 @@ ARDOUR::find_plugin(Session& session, string identifier, PluginType type)
 			return (*i)->load (session);
 		}
 	}
+
+#ifdef VST_SUPPORT
+	/* hmm, we didn't find it. could be because in older versions of Ardour.
+	   we used to store the name of a VST plugin, not its unique ID. so try
+	   again.
+	*/
+
+	for (i = plugs.begin(); i != plugs.end(); ++i) {
+		if (identifier == (*i)->name){
+			return (*i)->load (session);
+		}
+	}
+#endif
 	
 	return PluginPtr ((Plugin*) 0);
 }
