@@ -352,11 +352,8 @@ Region::set_position_internal (nframes_t pos, bool allow_bbt_recompute)
 			_length = max_frames - _position;
 		}
 
-		if (allow_bbt_recompute && _positional_lock_style == MusicTime) {
-			boost::shared_ptr<Playlist> pl (playlist());
-			if (pl) {
-				pl->session().tempo_map().bbt_time (_position, _bbt_time);
-			}
+		if (allow_bbt_recompute) {
+			recompute_position_from_lock_style ();
 		}
 
 		invalidate_transients ();
@@ -394,6 +391,17 @@ Region::set_position_on_top (nframes_t pos, void *src)
 	send_change (PositionChanged);
 }
 
+void
+Region::recompute_position_from_lock_style ()
+{
+	if (_positional_lock_style == MusicTime) {
+		boost::shared_ptr<Playlist> pl (playlist());
+		if (pl) {
+			pl->session().tempo_map().bbt_time (_position, _bbt_time);
+		}
+	}
+}
+		
 void
 Region::nudge_position (nframes64_t n, void *src)
 {
