@@ -35,8 +35,8 @@ sigc::signal<void,boost::shared_ptr<Region> > RegionFactory::CheckNewRegion;
 
 boost::shared_ptr<Region>
 RegionFactory::create (boost::shared_ptr<Region> region, nframes_t start, 
-			     nframes_t length, std::string name, 
-			     layer_t layer, Region::Flag flags, bool announce)
+		       nframes_t length, std::string name, 
+		       layer_t layer, Region::Flag flags, bool announce)
 {
 	boost::shared_ptr<const AudioRegion> other;
 
@@ -57,11 +57,11 @@ RegionFactory::create (boost::shared_ptr<Region> region, nframes_t start,
 }
 
 boost::shared_ptr<Region>
-RegionFactory::create (boost::shared_ptr<Region> region)
+RegionFactory::create (boost::shared_ptr<const Region> region)
 {
-	boost::shared_ptr<AudioRegion> other;
+	boost::shared_ptr<const AudioRegion> other;
 	
-	if ((other = boost::dynamic_pointer_cast<AudioRegion>(region)) != 0) {
+	if ((other = boost::dynamic_pointer_cast<const AudioRegion>(region)) != 0) {
 		boost::shared_ptr<Region> ret (new AudioRegion (other));
 		/* pure copy constructor - no CheckNewRegion emitted */
 		return ret;
@@ -75,8 +75,8 @@ RegionFactory::create (boost::shared_ptr<Region> region)
 
 boost::shared_ptr<Region>
 RegionFactory::create (boost::shared_ptr<AudioRegion> region, nframes_t start, 
-			     nframes_t length, std::string name, 
-			     layer_t layer, Region::Flag flags, bool announce)
+		       nframes_t length, std::string name, 
+		       layer_t layer, Region::Flag flags, bool announce)
 {
 	return create (boost::static_pointer_cast<Region> (region), start, length, name, layer, flags, announce);
 }
