@@ -24,6 +24,7 @@
 #include <sigc++/bind.h>
 #include <pbd/xml++.h>
 #include <pbd/enumwriter.h>
+#include <pbd/stacktrace.h>
 
 #include <ardour/timestamps.h>
 #include <ardour/buffer.h>
@@ -41,6 +42,7 @@
 #include <ardour/panner.h>
 #include <ardour/dB.h>
 #include <ardour/mix.h>
+#include <ardour/profile.h>
 
 #include "i18n.h"
 
@@ -184,6 +186,20 @@ Route::sync_order_keys ()
 	}
 }
 
+string
+Route::ensure_track_or_route_name(string name, Session &session)
+{
+	string newname = name;
+
+	while (session.route_by_name (newname)!=NULL)
+	{
+		newname = bump_name_once (newname);
+	}
+
+	return newname;
+}
+
+
 void
 Route::inc_gain (gain_t fraction, void *src)
 {
@@ -846,6 +862,14 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 		boost::shared_ptr<PluginInsert> pi;
 		boost::shared_ptr<PortInsert> porti;
 
+		_redirects.push_back (redirect);
+
+		if (_reset_plugin_counts (err_streams)) {
+			_redirects.pop_back ();
+			_reset_plugin_counts (0); // it worked before we tried to add it ...
+			return -1;
+		}
+
 		uint32_t potential_max_streams = 0;
 
 		if ((pi = boost::dynamic_pointer_cast<PluginInsert>(redirect)) != 0) {
@@ -886,14 +910,6 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 			_max_peak_power.push_back(-INFINITY);
 		}
 
-		_redirects.push_back (redirect);
-
-		if (_reset_plugin_counts (err_streams)) {
-			_redirects.pop_back ();
-			_reset_plugin_counts (0); // it worked before we tried to add it ...
-			return -1;
-		}
-
 		redirect->activate ();
 		redirect->active_changed.connect (mem_fun (*this, &Route::redirect_active_proxy));
 	}
@@ -902,8 +918,8 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 		reset_panner ();
 	}
 
-
 	redirects_changed (src); /* EMIT SIGNAL */
+
 	return 0;
 }
 
@@ -1114,14 +1130,14 @@ int
 Route::_reset_plugin_counts (uint32_t* err_streams)
 {
 	RedirectList::iterator r;
-	uint32_t i_cnt;
-	uint32_t s_cnt;
+	uint32_t insert_cnt = 0;
+	uint32_t send_cnt = 0;
 	map<Placement,list<InsertCount> > insert_map;
+	RedirectList::iterator prev;
 	nframes_t initial_streams;
+	int ret = -1;
 
 	redirect_max_outs = 0;
-	i_cnt = 0;
-	s_cnt = 0;
 
 	/* divide inserts up by placement so we get the signal flow
 	   properly modelled. we need to do this because the _redirects
@@ -1133,14 +1149,8 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 
 		boost::shared_ptr<Insert> insert;
 
-		/* do this here in case we bomb out before we get to the end of
-		   this function.
-		*/
-
-		redirect_max_outs = max ((*r)->output_streams (), redirect_max_outs);
-
 		if ((insert = boost::dynamic_pointer_cast<Insert>(*r)) != 0) {
-			++i_cnt;
+			++insert_cnt;
 			insert_map[insert->placement()].push_back (InsertCount (insert));
 
 			/* reset plugin counts back to one for now so
@@ -1155,15 +1165,16 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 			}
 
 		} else if (boost::dynamic_pointer_cast<Send> (*r) != 0) {
-			++s_cnt;
+			++send_cnt;
 		}
 	}
 	
-	if (i_cnt == 0) {
-		if (s_cnt) {
+	if (insert_cnt == 0) {
+		if (send_cnt) {
 			goto recompute;
 		} else {
-			return 0;
+			ret = 0;
+			goto streamcount;
 		}
 	}
 
@@ -1174,7 +1185,7 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	/* A: PreFader */
 	
 	if (check_some_plugin_counts (insert_map[PreFader], n_inputs (), err_streams)) {
-		return -1;
+		goto streamcount;
 	}
 
 	/* figure out the streams that will feed into PreFader */
@@ -1189,7 +1200,7 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	/* B: PostFader */
 
 	if (check_some_plugin_counts (insert_map[PostFader], initial_streams, err_streams)) {
-		return -1;
+		goto streamcount;
 	}
 
 	/* OK, everything can be set up correctly, so lets do it */
@@ -1202,7 +1213,7 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
   recompute:
 
 	redirect_max_outs = 0;
-	RedirectList::iterator prev = _redirects.end();
+	prev = _redirects.end();
 
 	for (r = _redirects.begin(); r != _redirects.end(); prev = r, ++r) {
 		boost::shared_ptr<Send> s;
@@ -1226,8 +1237,13 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	}
 
 	/* we're done */
-
 	return 0;
+
+  streamcount:
+	for (r = _redirects.begin(); r != _redirects.end(); ++r) {
+		redirect_max_outs = max ((*r)->output_streams (), redirect_max_outs);
+	}
+	return ret;
 }				   
 
 int32_t
@@ -2180,7 +2196,7 @@ Route::handle_transport_stopped (bool abort_ignored, bool did_locate, bool can_f
 		Glib::RWLock::ReaderLock lm (redirect_lock);
 
 		if (!did_locate) {
-			automation_snapshot (now);
+			automation_snapshot (now, true);
 		}
 
 		for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
@@ -2287,7 +2303,7 @@ Route::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nfra
 		if (lm.locked()) {
 			// automation snapshot can also be called from the non-rt context
 			// and it uses the redirect list, so we take the lock out here
-			automation_snapshot (_session.transport_frame());
+			automation_snapshot (_session.transport_frame(), false);
 		}
 	}
 
@@ -2428,12 +2444,16 @@ Route::set_latency_delay (nframes_t longest_session_latency)
 }
 
 void
-Route::automation_snapshot (nframes_t now)
+Route::automation_snapshot (nframes_t now, bool force)
 {
-	IO::automation_snapshot (now);
+	if (!force && !should_snapshot(now)) {
+		return;
+	}
+
+	IO::automation_snapshot (now, force);
 
 	for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
-		(*i)->automation_snapshot (now);
+		(*i)->automation_snapshot (now, force);
 	}
 }
 
