@@ -105,6 +105,7 @@ Session::apply_gain_to_buffer_t		Session::apply_gain_to_buffer 	= 0;
 Session::mix_buffers_with_gain_t	Session::mix_buffers_with_gain 	= 0;
 Session::mix_buffers_no_gain_t		Session::mix_buffers_no_gain 	= 0;
 
+sigc::signal<void,std::string> Session::Dialog;
 sigc::signal<int> Session::AskAboutPendingState;
 sigc::signal<int,nframes_t,nframes_t> Session::AskAboutSampleRateMismatch;
 sigc::signal<void> Session::SendFeedback;
@@ -113,6 +114,10 @@ sigc::signal<void> Session::SMPTEOffsetChanged;
 sigc::signal<void> Session::StartTimeChanged;
 sigc::signal<void> Session::EndTimeChanged;
 
+sigc::signal<void> Session::AutoBindingOn;
+sigc::signal<void> Session::AutoBindingOff;
+
+
 int
 Session::find_session (string str, string& path, string& snapshot, bool& isnew)
 {
@@ -577,14 +582,13 @@ Session::destroy ()
 
 		tmp = i;
 		++tmp;
-
+		
 		i->second->drop_references ();
-
+		
 		i = tmp;
 	}
-
 	audio_sources.clear ();
-
+	
 #ifdef TRACK_DESTRUCTION
 	cerr << "delete mix groups\n";
 #endif /* TRACK_DESTRUCTION */
@@ -667,9 +671,13 @@ Session::when_engine_running ()
 
 	/* we don't want to run execute this again */
 
+	BootMessage (_("Set block size and sample rate"));
+
 	set_block_size (_engine.frames_per_cycle());
 	set_frame_rate (_engine.frame_rate());
 
+	BootMessage (_("Using configuration"));
+
 	Config->map_parameters (mem_fun (*this, &Session::config_changed));
 
 	/* every time we reconnect, recompute worst case output latencies */
@@ -725,6 +733,8 @@ Session::when_engine_running ()
 		error << _("cannot setup Click I/O") << endmsg;
 	}
 
+	BootMessage (_("Compute I/O Latencies"));
+
 	set_worst_io_latencies ();
 
 	if (_clicking) {
@@ -735,6 +745,8 @@ Session::when_engine_running ()
 	   to the physical outputs currently available
 	*/
 
+	BootMessage (_("Set up standard connections"));
+
 	/* ONE: MONO */
 
 	for (uint32_t np = 0; np < n_physical_audio_outputs; ++np) {
@@ -839,11 +851,15 @@ Session::when_engine_running ()
 		}
 		add_connection (c);
 	} 
+	
+	BootMessage (_("Setup signal flow and plugins"));
 
 	hookup_io ();
 
 	/* catch up on send+insert cnts */
 
+	BootMessage (_("Catch up with send/insert state"));
+
 	insert_cnt = 0;
 	
 	for (list<PortInsert*>::iterator i = _port_inserts.begin(); i != _port_inserts.end(); ++i) {
@@ -871,14 +887,17 @@ Session::when_engine_running ()
 	
 	_state_of_the_state = StateOfTheState (_state_of_the_state & ~(CannotSave|Dirty));
 
-
 	/* hook us up to the engine */
 
+	BootMessage (_("Connect to engine"));
+
 	_engine.set_session (this);
 
 #ifdef HAVE_LIBLO
 	/* and to OSC */
 
+	BootMessage (_("OSC startup"));
+
 	osc->set_session (*this);
 #endif
     
@@ -1107,7 +1126,6 @@ Session::auto_loop_changed (Location* location)
 	}	
 
 	last_loopend = location->end();
-	
 }
 
 void
@@ -1145,6 +1163,10 @@ Session::set_auto_punch_location (Location* location)
 	auto_punch_changed_connection = location->changed.connect (mem_fun (this, &Session::auto_punch_changed));
 
 	location->set_auto_punch (true, this);
+
+
+	auto_punch_changed (location);
+
 	auto_punch_location_changed (location);
 }
 
@@ -1184,6 +1206,13 @@ Session::set_auto_loop_location (Location* location)
 	auto_loop_changed_connection = location->changed.connect (mem_fun (this, &Session::auto_loop_changed));
 
 	location->set_auto_loop (true, this);
+
+	/* take care of our stuff first */
+
+	auto_loop_changed (location);
+
+	/* now tell everyone else */
+
 	auto_loop_location_changed (location);
 }
 
@@ -1296,11 +1325,11 @@ Session::step_back_from_record ()
 	if (g_atomic_int_get (&_record_status) == Recording) {
 		g_atomic_int_set (&_record_status, Enabled);
 
-		if (Config->get_monitoring_model() == HardwareMonitoring) {
+		if (Config->get_monitoring_model() == HardwareMonitoring && Config->get_auto_input()) {
 			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
 			
 			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
-				if (Config->get_auto_input() && (*i)->record_enabled ()) {
+				if ((*i)->record_enabled ()) {
 					//cerr << "switching from input" << __FILE__ << __LINE__ << endl << endl;
 					(*i)->monitor_input (false);   
 				}
@@ -2567,6 +2596,14 @@ Session::region_name (string& result, string base, bool newlevel) const
 
 void
 Session::add_region (boost::shared_ptr<Region> region)
+{
+	vector<boost::shared_ptr<Region> > v;
+	v.push_back (region);
+	add_regions (v);
+}
+		
+void
+Session::add_regions (vector<boost::shared_ptr<Region> >& new_regions)
 {
 	boost::shared_ptr<AudioRegion> ar;
 	boost::shared_ptr<AudioRegion> oar;
@@ -2575,45 +2612,52 @@ Session::add_region (boost::shared_ptr<Region> region)
 	{ 
 		Glib::Mutex::Lock lm (region_lock);
 
-		if (region == 0) {
-			error << _("Session::add_region() ignored a null region. Warning: you might have lost a region.") << endmsg;
-		} else if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
+		for (vector<boost::shared_ptr<Region> >::iterator ii = new_regions.begin(); ii != new_regions.end(); ++ii) {
+		
+			boost::shared_ptr<Region> region = *ii;
+			
+			if (region == 0) {
 
-			AudioRegionList::iterator x;
-
-			for (x = audio_regions.begin(); x != audio_regions.end(); ++x) {
-
-				oar = boost::dynamic_pointer_cast<AudioRegion> (x->second);
-
-				if (ar->region_list_equivalent (oar)) {
-					break;
-				}
-			}
-
-			if (x == audio_regions.end()) {
-
-				pair<AudioRegionList::key_type,AudioRegionList::mapped_type> entry;
-
-				entry.first = region->id();
-				entry.second = ar;
-
-				pair<AudioRegionList::iterator,bool> x = audio_regions.insert (entry);
+				error << _("Session::add_region() ignored a null region. Warning: you might have lost a region.") << endmsg;
 
+			} else if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
 				
-				if (!x.second) {
-					return;
+				AudioRegionList::iterator x;
+				
+				for (x = audio_regions.begin(); x != audio_regions.end(); ++x) {
+					
+					oar = boost::dynamic_pointer_cast<AudioRegion> (x->second);
+					
+					if (ar->region_list_equivalent (oar)) {
+						break;
+					}
 				}
+				
+				if (x == audio_regions.end()) {
+					
+					pair<AudioRegionList::key_type,AudioRegionList::mapped_type> entry;
+					
+					entry.first = region->id();
+					entry.second = ar;
+					
+					pair<AudioRegionList::iterator,bool> x = audio_regions.insert (entry);
+					
+					
+					if (!x.second) {
+						return;
+					}
+					
+					added = true;
+				} 
 
-				added = true;
-			} 
-
-		} else {
-
-			fatal << _("programming error: ")
-			      << X_("unknown region type passed to Session::add_region()")
-			      << endmsg;
-			/*NOTREACHED*/
-
+			} else {
+				
+				fatal << _("programming error: ")
+				      << X_("unknown region type passed to Session::add_region()")
+				      << endmsg;
+				/*NOTREACHED*/
+				
+			}
 		}
 	}
 
@@ -2624,9 +2668,34 @@ Session::add_region (boost::shared_ptr<Region> region)
 	set_dirty();
 	
 	if (added) {
-		region->GoingAway.connect (sigc::bind (mem_fun (*this, &Session::remove_region), boost::weak_ptr<Region>(region)));
-		region->StateChanged.connect (sigc::bind (mem_fun (*this, &Session::region_changed), boost::weak_ptr<Region>(region)));
-		AudioRegionAdded (ar); /* EMIT SIGNAL */
+
+		vector<boost::weak_ptr<AudioRegion> > v;
+		boost::shared_ptr<AudioRegion> first_ar;
+
+		for (vector<boost::shared_ptr<Region> >::iterator ii = new_regions.begin(); ii != new_regions.end(); ++ii) {
+
+			boost::shared_ptr<Region> region = *ii;
+			boost::shared_ptr<AudioRegion> ar;
+
+			if (region == 0) {
+
+				error << _("Session::add_region() ignored a null region. Warning: you might have lost a region.") << endmsg;
+
+			} else if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
+				v.push_back (ar);
+
+				if (!first_ar) {
+					first_ar = ar;
+				}
+			}
+
+			region->StateChanged.connect (sigc::bind (mem_fun (*this, &Session::region_changed), boost::weak_ptr<Region>(region)));
+			region->GoingAway.connect (sigc::bind (mem_fun (*this, &Session::remove_region), boost::weak_ptr<Region>(region)));
+		}
+		
+		if (!v.empty()) {
+			AudioRegionsAdded (v); /* EMIT SIGNAL */
+		}
 	}
 }
 
@@ -4033,9 +4102,37 @@ Session::write_one_audio_track (AudioTrack& track, nframes_t start, nframes_t le
 vector<Sample*>&
 Session::get_silent_buffers (uint32_t howmany)
 {
+	if (howmany > _silent_buffers.size()) {
+
+		error << string_compose (_("Programming error: get_silent_buffers() called for %1 buffers but only %2 exist"),
+					 howmany, _silent_buffers.size()) << endmsg;
+
+		if (howmany > 1000) {
+			cerr << "ABSURD: more than 1000 silent buffers requested!\n";
+			abort ();
+		}
+		
+		while (howmany > _silent_buffers.size()) {
+			Sample *p = 0;
+			
+#ifdef NO_POSIX_MEMALIGN
+			p =  (Sample *) malloc(current_block_size * sizeof(Sample));
+#else
+			if (posix_memalign((void **)&p,CPU_CACHE_ALIGN,current_block_size * 4) != 0) {
+				fatal << string_compose (_("Memory allocation error: posix_memalign (%1 * %2) failed (%3)"),
+							 current_block_size, sizeof (Sample), strerror (errno))
+				      << endmsg;
+				/*NOTREACHED*/
+			}
+#endif			
+			_silent_buffers.push_back (p);
+		}
+	}
+
 	for (uint32_t i = 0; i < howmany; ++i) {
 		memset (_silent_buffers[i], 0, sizeof (Sample) * current_block_size);
 	}
+
 	return _silent_buffers;
 }
 
