@@ -335,11 +335,15 @@ Session::second_stage_init (bool new_session)
 		return -1;
 	}
 
+	BootMessage (_("Reset Remote Controls"));
+
 	send_full_time_code ();
 	_engine.transport_locate (0);
 	deliver_mmc (MIDI::MachineControl::cmdMmcReset, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, 0);
 
+	BootMessage (_("Reset Control Protocols"));
+
 	ControlProtocolManager::instance().set_session (*this);
 
 	if (new_session) {
@@ -349,7 +353,6 @@ Session::second_stage_init (bool new_session)
 	}
 
 	_state_of_the_state = Clean;
-
 	
 	DirtyChanged (); /* EMIT SIGNAL */
 
@@ -359,6 +362,8 @@ Session::second_stage_init (bool new_session)
 		state_was_pending = false;
 	}
 	
+	BootMessage (_("Session loading complete"));
+
 	return 0;
 }
 
@@ -1380,6 +1385,8 @@ Session::load_routes (const XMLNode& node)
 			return -1;
 		}
 
+		BootMessage (string_compose (_("Loaded track/bus %1"), route->name()));
+
 		new_routes.push_back (route);
 	}
 
@@ -1631,7 +1638,8 @@ Session::XMLSourceFactory (const XMLNode& node)
 	}
 
 	try {
-		return SourceFactory::create (*this, node);
+		/* note: do peak building in another thread when loading session state */
+		return SourceFactory::create (*this, node, true);
 	}
 
 	catch (failed_constructor& err) {
