@@ -29,6 +29,7 @@
 #include <glibmm/thread.h>
 #include <pbd/pthread_utils.h>
 #include <pbd/memento_command.h>
+#include <pbd/stacktrace.h>
 
 #include <midi++/mmc.h>
 #include <midi++/port.h>
@@ -54,7 +55,7 @@ Session::request_input_change_handling ()
 	if (!(_state_of_the_state & (InitialConnecting|Deletion))) {
 		Event* ev = new Event (Event::InputConfigurationChange, Event::Add, Event::Immediate, 0, 0.0);
 		queue_event (ev);
-	}
+	} 
 }
 
 void
@@ -371,17 +372,37 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		update_latency_compensation (true, abort);
 	}
 
-	if ((Config->get_slave_source() == None && Config->get_auto_return()) || (post_transport_work & PostTransportLocate) || synced_to_jack()) {
+	if ((Config->get_slave_source() == None && Config->get_auto_return()) || 
+	    (post_transport_work & PostTransportLocate) || 
+	    (_requested_return_frame >= 0) ||
+	    synced_to_jack()) {
 		
 		if (pending_locate_flush) {
 			flush_all_redirects ();
 		}
 		
-		if (((Config->get_slave_source() == None && Config->get_auto_return()) || synced_to_jack()) && !(post_transport_work & PostTransportLocate)) {
+		if (((Config->get_slave_source() == None && Config->get_auto_return()) || 
+		     synced_to_jack() ||
+		     _requested_return_frame >= 0) &&
+		    !(post_transport_work & PostTransportLocate)) {
 
-			_transport_frame = last_stop_frame;
+			bool do_locate = false;
 
-			if (synced_to_jack()) {
+			if (_requested_return_frame >= 0) {
+				_transport_frame = _requested_return_frame;
+				_requested_return_frame = -1;
+				do_locate = true;
+			} else {
+				_transport_frame = last_stop_frame;
+				_requested_return_frame = -1;
+			}
+
+			if (synced_to_jack() && !play_loop) {
+				do_locate = true;
+			}
+			
+			if (do_locate) {
+				// cerr << "non-realtimestop: transport locate to " << _transport_frame << endl;
 				_engine.transport_locate (_transport_frame);
 			}
 		} 
@@ -409,9 +430,14 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 	}
 #endif
 
-	last_stop_frame = _transport_frame;
+        if (_requested_return_frame < 0) {
+		last_stop_frame = _transport_frame;
+	} else {
+		last_stop_frame = _requested_return_frame;
+		_requested_return_frame = -1;
+	}
 
-	send_full_time_code ();
+        send_full_time_code ();
 	deliver_mmc (MIDI::MachineControl::cmdStop, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, _transport_frame);
 
@@ -420,14 +446,16 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		/* XXX its a little odd that we're doing this here
 		   when realtime_stop(), which has already executed,
 		   will have done this.
+		   JLC - so let's not because it seems unnecessary and breaks loop record
 		*/
-		
+#if 0
 		if (!Config->get_latched_record_enable()) {
 			g_atomic_int_set (&_record_status, Disabled);
 		} else {
 			g_atomic_int_set (&_record_status, Enabled);
 		}
 		RecordStateChanged (); /* emit signal */
+#endif
 	}
 	
 	if ((post_transport_work & PostTransportLocate) && get_record_enabled()) {
@@ -457,7 +485,9 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		
 	}
 
-	PositionChanged (_transport_frame); /* EMIT SIGNAL */
+        nframes_t tf = _transport_frame;
+
+        PositionChanged (tf); /* EMIT SIGNAL */
 	TransportStateChange (); /* EMIT SIGNAL */
 
 	/* and start it up again if relevant */
@@ -613,7 +643,7 @@ Session::start_locate (nframes_t target_frame, bool with_roll, bool with_flush,
 void
 Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
 {
-	if (actively_recording()) {
+	if (actively_recording() && !with_loop) {
 		return;
 	}
 
@@ -704,6 +734,22 @@ Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool w
 			// cancel looping directly, this is called from event handling context
 			set_play_loop (false);
 		}
+		else if (al && _transport_frame == al->start()) {
+			if (with_loop) {
+				// this is only necessary for seamless looping
+
+				boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+				
+				for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+					if ((*i)->record_enabled ()) {
+						// tell it we've looped, so it can deal with the record state
+						(*i)->transport_looped(_transport_frame);
+					}
+				}
+			}
+
+			TransportLooped(); // EMIT SIGNAL
+		}
 	}
 	
 	loop_changing = false;
@@ -874,7 +920,9 @@ Session::start_transport ()
 		break;
 
 	case Recording:
-		disable_record (false);
+		if (!play_loop) {
+			disable_record (false);
+		}
 		break;
 
 	default:
@@ -1143,6 +1191,14 @@ Session::setup_auto_play ()
 	merge_event (ev);
 }
 
+void
+Session::request_roll_at_and_return (nframes_t start, nframes_t return_to)
+{
+ 	Event *ev = new Event (Event::LocateRollLocate, Event::Add, Event::Immediate, return_to, 1.0);
+	ev->target2_frame = start;
+	queue_event (ev);
+}
+
 void
 Session::request_bounded_roll (nframes_t start, nframes_t end)
 {
@@ -1179,9 +1235,9 @@ Session::engine_halted ()
 void
 Session::xrun_recovery ()
 {
-	if (Config->get_stop_recording_on_xrun() && actively_recording()) {
+	Xrun (transport_frame()); //EMIT SIGNAL
 
-		 HaltOnXrun (); /* EMIT SIGNAL */
+	if (Config->get_stop_recording_on_xrun() && actively_recording()) {
 
 		/* it didn't actually halt, but we need
 		   to handle things in the same way.
